Package: plotbb
Title: Grammar of Graphics for 'base' Plot
Version: 0.1.2
Authors@R: c(
	   person("Guangchuang", "Yu",     email = "guangchuangyu@gmail.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-6485-8781"))
     )
Description: A grammar of graphics framework built on 'base' graphics. It
  provides a 'bbplot' object and a '+' operator to incrementally compose plots
  from data, aesthetic mappings and layers, then render them using the base
  plotting system. The package includes common geometric layers (points, lines,
  segments, bars, histograms, boxplots and tiles), scales for color and other
  aesthetics, legends, faceting, themes, and significance annotations.
Imports: graphics, magrittr, rlang, scales, methods, utils
Suggests: stats, ape, aplot (>= 0.0.8), dplyr, ggplotify (>= 0.1.0),
        quarto, knitr, testthat
VignetteBuilder: quarto
ByteCompile: true
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-21 04:50:56 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 05:00:02 UTC
Built: R 4.5.2; ; 2026-05-25 01:22:53 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("+",bbplot)
S3method("<=",bb_theme)
S3method(bbplot_add,"function")
S3method(bbplot_add,bb_facet_wrap)
S3method(bbplot_add,bb_grid)
S3method(bbplot_add,bb_labs)
S3method(bbplot_add,bb_layer)
S3method(bbplot_add,bb_legend)
S3method(bbplot_add,bb_palette)
S3method(bbplot_add,bb_scale)
S3method(bbplot_add,bb_theme)
S3method(bbplot_add,expression)
S3method(bbplot_add,formula)
S3method(print,bbplot)
S3method(print,bbplot_layer_list)
export("%>%")
export(as.bbplot)
export(bb_aes)
export(bb_bar)
export(bb_boxplot)
export(bb_density)
export(bb_errorbar)
export(bb_facet_wrap)
export(bb_grid)
export(bb_hist)
export(bb_labs)
export(bb_legend)
export(bb_line)
export(bb_lm)
export(bb_point)
export(bb_scale_cex_continuous)
export(bb_scale_cex_manual)
export(bb_scale_col_gradient)
export(bb_scale_col_manual)
export(bb_scale_col_palette)
export(bb_scale_lty_manual)
export(bb_scale_pch_manual)
export(bb_segment)
export(bb_signif)
export(bb_sub)
export(bb_text)
export(bb_theme)
export(bb_theme_bw)
export(bb_theme_deepblue)
export(bb_theme_expand)
export(bb_theme_grey)
export(bb_theme_minimal)
export(bb_tile)
export(bb_title)
export(bb_xlab)
export(bb_ylab)
export(bbplot)
export(bbplot_add)
export(set_bb_theme)
export(unset_bb_theme)
importFrom(graphics,boxplot)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(rlang,enquos)
importFrom(rlang,quo_name)
importFrom(scales,col_factor)
importFrom(scales,col_numeric)
importFrom(stats,density)
importFrom(utils,modifyList)

## -----------------------------------------------------------------------------
#| label: setup
#| include: false
r_libs <- Sys.getenv("R_LIBS")
if (nzchar(r_libs)) {
  libs <- strsplit(r_libs, .Platform$path.sep, fixed = TRUE)[[1]]
  libs <- libs[nzchar(libs)]
  .libPaths(c(libs, .libPaths()))
}
library(dplyr)
library(plotbb)


## ----plotbb-aes---------------------------------------------------------------
library(plotbb)

p <- bbplot(mtcars, bb_aes(mpg, disp, col = factor(cyl)))
p + bb_grid(col = "grey50", lty = "dashed") + bb_point(pch = 19)


## ----plotbb-layer-------------------------------------------------------------
#| fig-height: 4
p2 <- p + bb_point() + bb_lm(bb_aes(group = cyl), lwd = 2)
p3 <- p2 + bb_lm(col = "red", lwd = 3, lty = "dotted")
p4 <- p + bb_text(bb_aes(label = cyl), cex = 2)
aplot::plot_list(p2, p3, p4, ncol = 3, tag_levels = "A")


## ----plotbb-line-seg-errorbar-------------------------------------------------
d2 <- group_by(mtcars, cyl) %>%
  summarize(
    x = cyl[1],
    y = mean(disp),
    sd = sd(disp)
  )

g1 <- bbplot(d2, bb_aes(x, y)) +
  bb_line(col = "grey50") +
  bb_point(pch = 19, cex = 1.5) +
  bb_errorbar(bb_aes(ymin = y - sd, ymax = y + sd), width = 0.8)

seg <- data.frame(x = 15, y = 150, xend = 30, yend = 450)
g2 <- bbplot(seg, bb_aes(x, y)) +
  bb_point(pch = 19) +
  bb_segment(bb_aes(xend = xend, yend = yend), lwd = 3, col = "steelblue")

aplot::plot_list(g1, g2, ncol = 2, tag_levels = "A")


## ----plotbb-bar---------------------------------------------------------------
mtcars$cyl <- factor(mtcars$cyl, levels = c(8, 4, 6))
bbplot(mtcars, bb_aes(cyl, col = cyl)) +
  bb_bar(stat = "count") +
  bb_legend(position = "topleft")


## ----plotbb-bar-identity------------------------------------------------------
dbar <- group_by(mtcars, cyl) %>%
  summarize(mpg = mean(mpg), .groups = "drop")
bbplot(dbar, bb_aes(cyl, mpg, col = cyl)) +
  bb_bar() +
  bb_legend(position = "topleft")


## ----plotbb-boxplot-----------------------------------------------------------
bbplot(mtcars, bb_aes(factor(cyl), mpg)) +
  bb_boxplot(fill = "grey90")


## ----plotbb-hist--------------------------------------------------------------
bbplot(mtcars, bb_aes(mpg)) +
  bb_hist(col = "grey80", border = "white")


## ----plotbb-signif------------------------------------------------------------
bbplot(mtcars, bb_aes(factor(cyl), mpg)) +
  bb_boxplot(fill = "grey90") +
  bb_signif(comparisons = list(c("4", "6"), c("4", "8")))


## ----plotbb-heatmap-----------------------------------------------------------
#| fig-height: 12
df <- data.frame(
  x = rep(1:10, 12),
  y = rep(1:12, each = 10),
  values = rnorm(120, mean = 10, sd = 5),
  type = sample(LETTERS[1:5], 120, replace = TRUE),
  stringsAsFactors = FALSE
)

h1 <- bbplot(df, bb_aes(x, y, col = values)) + bb_tile() +
  bb_title("heatmap for continuous numerical values")
h2 <- bbplot(df, bb_aes(x, y, col = values)) + bb_tile() + bb_scale_col_palette("YlOrRd") +
  bb_title("applying a color palette")
h3 <- bbplot(df, bb_aes(x, y, col = type)) + bb_tile() +
  bb_title("heatmap for discrete categorical values")
h4 <- bbplot(df, bb_aes(x, y, col = values)) + bb_tile() + bb_text(col = "black") +
  bb_title("heatmap with text labels") + bb_theme_expand()
aplot::plot_list(h1, h2, h3, h4, ncol = 2, tag_levels = "A")


## ----plotbb-labs--------------------------------------------------------------
p2 + bb_labs(
  title = "hello",
  sub = "just for demo",
  xlab = "this is xlab",
  ylab = "this is ylab"
) +
  bb_title("hello world")


## ----plotbb-theme-------------------------------------------------------------
#| fig-height: 6
g <- p2 +
  bb_theme(
    col.main = "red",
    cex.main = 2,
    mar = c(4, 4, 3, 1)
  ) +
  bb_title("applying graphics::par")
g2 <- p2 + bb_title("theme has no side effect")
aplot::plot_list(g, g2, ncol = 2, tag_levels = "A")


## ----plotbb-theme-expand------------------------------------------------------
#| fig-height: 6
p4 <- p3 + bb_theme_expand()
aplot::plot_list(p4, p3, ncol = 2, tag_levels = "A")


## ----plotbb-theme-grey--------------------------------------------------------
p + bb_grid(col = "grey50", lty = "dashed") +
  bb_point(pch = 19) +
  bb_theme_expand() +
  bb_theme_grey()


## ----plotbb-theme-blue--------------------------------------------------------
p + bb_point(pch = 19, cex = 2) +
  bb_theme_expand() +
  bb_theme_deepblue()


## ----plotbb-theme-bw-minimal--------------------------------------------------
#| fig-height: 6
g_bw <- p2 + bb_theme_expand() + bb_theme_bw() + bb_title("bb_theme_bw")
g_min <- p2 + bb_theme_expand() + bb_theme_minimal() + bb_title("bb_theme_minimal")
aplot::plot_list(g_bw, g_min, ncol = 2, tag_levels = "A")


## ----plotbb-scale-manual------------------------------------------------------
pal <- c(`4` = "#1b9e77", `6` = "#d95f02", `8` = "#7570b3")
bbplot(mtcars, bb_aes(mpg, disp, col = factor(cyl))) +
  bb_point(pch = 19) +
  bb_scale_col_manual(pal) +
  bb_legend()


## ----plotbb-scale-gradient----------------------------------------------------
bbplot(mtcars, bb_aes(mpg, disp, col = hp)) +
  bb_point(pch = 19) +
  bb_scale_col_gradient(low = "grey80", high = "firebrick") +
  bb_legend()


## ----plotbb-scale-aes---------------------------------------------------------
#| fig-width: 16
p5 <- bbplot(mtcars, bb_aes(mpg, disp, col = factor(cyl), pch = factor(gear), cex = hp)) +
  bb_point() +
  bb_scale_cex_continuous(range = c(0.8, 2)) +
  bb_legend(aesthetic = "col", position = "topleft") +
  bb_legend(aesthetic = "pch", position = "bottomleft")

p6 <- bbplot(mtcars, bb_aes(mpg, disp, lty = factor(gear))) +
  bb_lm(lwd = 2) +
  bb_scale_lty_manual(values = c(`3` = "solid", `4` = "dashed", `5` = "dotted")) +
  bb_legend(aesthetic = "lty", position = "topleft")

aplot::plot_list(p5, p6, ncol = 2, tag_levels = "A")


## ----plotbb-legend------------------------------------------------------------
p + bb_point(pch = 19) + bb_legend()


## ----plotbb-facet-------------------------------------------------------------
#| fig-height: 4
p + bb_point(pch = 19) + bb_facet_wrap(~cyl, ncol = 3)


## ----plotbb-facet-free--------------------------------------------------------
#| fig-height: 4
p + bb_point(pch = 19) + bb_facet_wrap(~cyl, ncol = 3, scales = "free_y")


## -----------------------------------------------------------------------------
f <- function() {
  plot(mtcars$mpg, mtcars$disp)
  abline(lm(disp ~ mpg, data = mtcars), col = "red")
}


## -----------------------------------------------------------------------------
library(dplyr)
d <- group_by(mtcars, cyl) %>%
  summarize(xm = mean(mpg), ym = mean(disp))

pp <- as.bbplot(f) +
  bb_theme_expand() +
  bb_theme_grey() +
  bb_lm(bb_aes(mpg, disp, group = cyl, col = factor(cyl)), data = mtcars, lwd = 2, lty = "dashed") +
  bb_point(bb_aes(xm, ym, col = factor(cyl)), data = d, pch = 19, cex = 2) +
  bb_title("hello plotbb") +
  bb_grid(col = "grey30", lty = "dashed")


## ----plotbb-base--------------------------------------------------------------
pp + (~points(30, 400, pch = 19, col = "red", cex = 3)) +
  ~text(30, 420, label = "have fun :)", col = "blue", cex = 1.2)


## ----plotbb-ape---------------------------------------------------------------
#| fig-height: 8.5
require(ape)
set.seed(2020 - 09 - 10)
x = rtree(10)

p = as.bbplot(
  function()
    plot(
      x,
      cex = 2,
      edge.width = 2,
      edge.color = "white",
      tip.color = "purple"
    )
)

p + bb_theme_expand() + bb_theme_deepblue() +
  (~nodelabels(node = 14, pie = matrix(rep(1, 4), 1), cex = 2)) +
  (~nodelabels(node = 15, pie = matrix(rep(1, 3), 1), cex = 2)) +
  (~nodelabels(node = 18, pie = matrix(rep(1, 5), 1), cex = 2)) +
  bb_labs(title = "phylogenetic tree plotted by base graphics") +
  bb_theme(col.main = "firebrick", cex.main = 2)

##' add grid lines
##'
##' 
##' @title bb_grid
##' @rdname bb-grid
##' @param col line color
##' @param lty line type
##' @param lwd line width
##' @param bg whether plot the grid lines as background
##' @return A modified bbplot object
##' @export
##' @author Guangchuang Yu
bb_grid <- function(col = 'lightgray', lty = par("lty"), lwd = par("lwd"), bg = TRUE) {
    structure(list(col = col,
                   lty = lty,
                   lwd = lwd,
                   bg = bg
                   ),
              class = "bb_grid")
}

##' @importFrom graphics grid
ly_grid <- function(plot, col, lty, lwd) {
    ly <- function() {
        grid(NULL, NULL, col = col, lty = lty, lwd = lwd)        
    }
    add_layer(plot, ly, "background grid line")
}

##' @method bbplot_add bb_grid
##' @export
bbplot_add.bb_grid <- function(object, plot) {
    bg <- object$bg
    object <- object[names(object) != 'bg']
    object$nx <- NULL
    object$ny <- NULL
    ly <- function() do.call(grid, object)
    if (bg) {
        ## plot$canvas <- function() plot$canvas(panel.first = ly)
        plot$panel.first <- ly
        return(plot)
    }
    add_layer(plot, ly, "background grid line")
}

##' boxplot layer
##'
##' @title bb_boxplot
##' @rdname layer
##' @param mapping aesthetic mapping
##' @param data layer data
##' @param ... addition parameter for the layer
##' @return A modified bbplot object
##' @importFrom graphics boxplot
##' @export
##' @author Guangchuang Yu
bb_boxplot <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ..., layer = ly_boxplot)
}

ly_boxplot <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    params <- list(...)
    if (is.null(params$add)) params$add <- TRUE
    if (is.null(params$axes)) params$axes <- FALSE
    
    ly <- function() {
        if (is.null(mapping$y)) {
            # If no y is mapped, check if x is mapped (for single variable boxplot? or should we enforce y?)
            # ggplot2 uses y for the variable in boxplot usually, or x/y combo.
            # base boxplot(x) plots x. 
            # If mapping$x is present and mapping$y is NULL, it's ambiguous.
            # Let's assume standard ggplot2:
            # aes(group, value) -> boxplot(value ~ group)
            # aes(value) -> boxplot(value) ?? In ggplot2 aes(y=value) gives one boxplot.
            
            # Let's support:
            # 1. aes(x, y) -> boxplot(y ~ x)
            # 2. aes(y) -> boxplot(y)
            return(invisible(NULL))
        }

        y <- bb_eval_one(mapping$y, data)
        
        args <- params
        
        if (!is.null(mapping$col)) {
             col_vec <- bb_col(mapping, data, plot = plot)
             # simple case: if col maps to the same group as x, it works naturally if we pass a vector
             # But boxplot(..., col=...) expects one color per group usually.
             
             # If mapping$col is same as mapping$x, we can extract unique colors.
             # This is a bit complex for base boxplot.
             # Let's pass the full vector if possible, or leave it to user to ensure it matches.
             # base boxplot col argument: "vector of colors (one for each group)"
             
             # For now, let's try passing it directly, but mapped colors are usually per-observation.
             # We might need to aggregate.
             
             if (!is.null(mapping$x)) {
                 x <- bb_eval_one(mapping$x, data)
                 # We need to find the color for each level of x.
                 # Assuming 1:1 mapping between x level and col.
                 udata <- unique(data.frame(x=as.character(x), col=col_vec))
                 # Sort by factor level of x if x is factor?
                 if (is.factor(x)) {
                     levs <- levels(x)
                     # Filter udata to only include levels present
                     udata <- udata[udata$x %in% levs, ]
                     # This is getting complicated. 
                     # Simplifying assumption: user passes a palette or single color via ... 
                     # OR if they map 'col', we rely on base plot behavior? 
                     # Base plot behavior for `col` in formula interface:
                     # "if there are fewer colors than boxes, they are recycled."
                     
                     # Let's try to infer if we can get a color-per-group.
                     # If not, ignore mapping$col or pass it as is (which might fail or look wrong).
                 }
                 
                 # Let's pass 'col' into args if it's not in there already?
                 # Actually, `col` is already in `params` if user set it manually.
                 # Here we are handling `mapping$col`.
                 
                 args$col <- col_vec # This might be wrong if length(col_vec) == nrow(data)
             } else {
                 # No x, just y. 
                 args$col <- unique(col_vec)[1] # Take first color?
             }
        }


        if (!is.null(mapping$x)) {
            f <- stats::as.formula(paste(rlang::as_label(mapping$y), "~", rlang::as_label(mapping$x)))
            do.call(graphics::boxplot, c(list(formula = f, data = data), args))
            return(invisible(NULL))
        }

        do.call(graphics::boxplot, c(list(x = y), args))
    }
    
    add_layer(plot, ly, "boxplot layer")
}

test_that("bbplot basic execution", {
  p <- bbplot(mtcars, bb_aes(mpg, disp))
  expect_s3_class(p, "bbplot")
})

test_that("bbplot with layers", {
  p <- bbplot(mtcars, bb_aes(mpg, disp)) + bb_point()
  expect_true(length(p$layer) == 1)
})

test_that("bb_boxplot execution", {
  p <- bbplot(mtcars, bb_aes(factor(cyl), mpg)) + bb_boxplot()
  expect_s3_class(p, "bbplot")
  expect_true("boxplot layer" %in% names(p$layer))
})

test_that("bb_boxplot defaults to add=TRUE and axes=FALSE", {
  p <- bbplot(mtcars, bb_aes(factor(cyl), mpg)) + bb_boxplot()
  ly <- p$layer[["boxplot layer"]]
  expect_true(is.function(ly))
  env <- environment(ly)
  expect_true(is.list(env$params))
  expect_identical(env$params$add, TRUE)
  expect_identical(env$params$axes, FALSE)
})

test_that("bb_hist execution", {
  p <- bbplot(mtcars, bb_aes(x = mpg)) + bb_hist()
  expect_s3_class(p, "bbplot")
  expect_true("histogram layer" %in% names(p$layer))
})

test_that("bb_hist contributes to plot limits", {
  p <- bbplot(mtcars, bb_aes(mpg)) + bb_hist()
  lim <- plotbb:::bb_plot_limits(p)
  expect_true(is.numeric(lim$xlim) && length(lim$xlim) == 2)
  expect_true(is.numeric(lim$ylim) && length(lim$ylim) == 2)
  expect_true(is.finite(lim$xlim[[1]]) && is.finite(lim$xlim[[2]]))
  expect_true(lim$ylim[[2]] > 0)
})

test_that("bb_signif comparisons execution", {
  p <- bbplot(mtcars, bb_aes(factor(cyl), mpg)) +
    bb_boxplot() +
    bb_signif(comparisons = list(c("4", "6")))
  expect_s3_class(p, "bbplot")
  expect_true("signif layer" %in% names(p$layer))

  lim <- plotbb:::bb_plot_limits(p)
  expect_true(lim$ylim[[2]] > max(mtcars$mpg, na.rm = TRUE))
})

test_that("bb_signif wilcox ties warning is suppressed by default", {
  df <- data.frame(
    g = factor(rep(c("A", "B"), each = 10)),
    y = c(rep(1, 5), rep(2, 5), rep(1, 5), rep(2, 5))
  )
  p <- bbplot(df, bb_aes(g, y)) +
    bb_boxplot() +
    bb_signif(comparisons = list(c("A", "B")))

  f <- tempfile(fileext = ".pdf")
  grDevices::pdf(f)
  on.exit(grDevices::dev.off(), add = TRUE)
  expect_warning(print(p), NA)
})

test_that("bb_bar canvas respects global limits", {
  df <- data.frame(g = factor(rep(c("A", "B"), each = 5)), y = c(1:5, 2:6))
  p <- bbplot(df, bb_aes(g, y)) +
    bb_bar() +
    bb_signif(comparisons = list(c("A", "B")))

  fml <- formals(p$canvas)
  expect_true(all(c("xlim", "ylim") %in% names(fml)))

  lim <- plotbb:::bb_plot_limits(p)
  expect_true(lim$ylim[[2]] > max(df$y, na.rm = TRUE))
})

test_that("faceting prints with panel strips and global title", {
  p <- bbplot(iris, bb_aes(Petal.Length, Sepal.Length, col = Species)) +
    bb_point(pch = 19) +
    bb_facet_wrap(~Species, nrow = 1) +
    bb_title("Overall title")

  f <- tempfile(fileext = ".pdf")
  grDevices::pdf(f)
  on.exit(grDevices::dev.off(), add = TRUE)
  expect_warning(print(p), NA)
})

test_that("grouped bb_lm skips CI warning for tiny groups", {
  df <- data.frame(
    x = c(1, 2, 3, 4),
    y = c(2, 4, 3, 5),
    g = c("A", "A", "B", "B")
  )
  p <- bbplot(df, bb_aes(x, y, col = g)) +
    bb_point(pch = 19) +
    bb_lm(bb_aes(group = g), se = TRUE)

  f <- tempfile(fileext = ".pdf")
  grDevices::pdf(f)
  on.exit(grDevices::dev.off(), add = TRUE)
  expect_warning(print(p), NA)
})

##' change labels for bbplot
##'
##' setting one or several of 'title', 'sub', 'xlab', and 'ylab'
##' @title bb_labs
##' @rdname bblabs
##' @param title title
##' @param sub sub
##' @param xlab xlab
##' @param ylab ylab
##' @return A modified bbplot object 
##' @importFrom graphics title
##' @export
##' @examples
##' library(plotbb)
##' p <- bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl))) + bb_point(pch=19)
##' p + bb_labs(title = "hello", sub = "just for demo",
##'             xlab="this is xlab", ylab = "this is ylab") +
##'     bb_title("hello world") # last one rules
##' @author Guangchuang Yu
bb_labs <- function(title = NULL, sub = NULL, xlab = NULL, ylab = NULL) {
    labs <- list(main = title,
                 sub = sub,
                 xlab = xlab,
                 ylab = ylab
                 )
    labs <- labs[!vapply(labs, is.null, logical(1))]
    structure(labs, class = "bb_labs")
}

##' @rdname bblabs
##' @export
bb_title <- function(title) {
    bb_labs(title = title)
}

##' @rdname bblabs
##' @export
bb_sub <- function(sub) {
    bb_labs(sub = sub)
}

##' @rdname bblabs
##' @export
bb_xlab <- function(xlab) {
    bb_labs(xlab = xlab)
}

##' @rdname bblabs
##' @export
bb_ylab <- function(ylab) {
    bb_labs(ylab = ylab)
}


##' @method bbplot_add bb_labs
##' @export
bbplot_add.bb_labs <- function(object, plot) {
    plot$labs <- modifyList(plot$labs, object)
    plot
}

.onAttach <- function(libname, pkgname) {
    .plotbb_env_initial()
}


.plotbb_env_initial <- function() {
    pos <- 1
    envir <- as.environment(pos)
    assign(".bbplot", new.env(), envir = envir)
}

##' @method print bbplot
##' @export
print.bbplot <- function(x, ...) {
    if (!is.null(x$facet)) {
        old.par <- par(no.readonly=TRUE)
        on.exit(suppressWarnings(par(old.par, no.readonly = TRUE)))

        if (length(x$theme)) {
            par(x$theme, no.readonly = TRUE)
        }

        facet <- x$facet
        data <- x$data
        if (is.null(data)) stop("bb_facet_wrap requires plot data")

        facet_val <- rlang::eval_tidy(rlang::f_rhs(facet$facets), data = data)
        facet_chr <- as.character(facet_val)
        facet_chr <- facet_chr[!is.na(facet_chr)]
        lev <- sort(unique(facet_chr))
        if (length(lev) == 0) return(invisible(NULL))

        lim_global <- bb_plot_limits(x)

        scales <- facet$scales %||% "fixed"
        scales <- match.arg(scales, c("fixed", "free", "free_x", "free_y"))

        if (!is.null(facet$nrow) && is.null(facet$ncol)) {
            nrow <- facet$nrow
            ncol <- ceiling(length(lev) / nrow)
        } else if (is.null(facet$nrow) && !is.null(facet$ncol)) {
            ncol <- facet$ncol
            nrow <- ceiling(length(lev) / ncol)
        } else if (!is.null(facet$nrow) && !is.null(facet$ncol)) {
            nrow <- facet$nrow
            ncol <- facet$ncol
        } else {
            ncol <- ceiling(sqrt(length(lev)))
            nrow <- ceiling(length(lev) / ncol)
        }

        global_main <- x$labs$main
        global_sub <- x$labs$sub
        top_oma <- if (!is.null(global_main) && !is.null(global_sub)) {
            4
        } else if (!is.null(global_main) || !is.null(global_sub)) {
            3
        } else {
            0
        }

        adds <- x$adds
        outside_legends <- Filter(function(obj) inherits(obj, "bb_legend") && isTRUE(obj$outside), adds)
        if (length(outside_legends)) {
            panel_layout <- matrix(seq_len(nrow * ncol), nrow = nrow, ncol = ncol, byrow = TRUE)
            legend_idx <- max(panel_layout) + 1
            layout_mat <- cbind(panel_layout, rep(legend_idx, nrow))
            graphics::layout(layout_mat, widths = c(rep(1, ncol), 0.38))
            graphics::par(oma = c(0, 0, top_oma, 0))
        } else {
            par(mfrow = c(nrow, ncol), oma = c(0, 0, top_oma, 0))
        }

        for (i in seq_along(lev)) {
            v <- lev[i]
            idx <- as.character(facet_val) == v
            d <- data[idx, , drop = FALSE]

            base_canvas <- function(xlim = NULL, ylim = NULL) {
                xy <- bb_eval_xy(x$mapping, d)
                if (!is.null(xy$x) && !is.null(xy$y)) {
                    args <- list(xy$x, xy$y, type = "n", xlab = "", ylab = "")
                    if (!is.null(xlim)) args$xlim <- xlim
                    if (!is.null(ylim)) args$ylim <- ylim
                    if (nrow == 1 && scales %in% c("fixed", "free_x") && i > 1) {
                        args$yaxt <- "n"
                    }
                    do.call(graphics::plot, args)
                    return(invisible(NULL))
                }
                bb_call_canvas(x$canvas, xlim = xlim, ylim = ylim)
                invisible(NULL)
            }

            p <- .bbplot_initial(base_canvas, d, x$mapping)

            if (length(adds)) {
                for (obj in adds) {
                    if (inherits(obj, "bb_facet_wrap")) next
                    if (inherits(obj, "bb_legend") && isTRUE(obj$outside)) next
                    p <- p + obj
                }
            }

            if (identical(scales, "fixed")) {
                lim <- lim_global
            } else {
                lim_panel <- bb_plot_limits(p)
                lim <- list(
                    xlim = if (scales %in% c("free", "free_x")) lim_panel$xlim else lim_global$xlim,
                    ylim = if (scales %in% c("free", "free_y")) lim_panel$ylim else lim_global$ylim
                )
            }

            p$labs$main <- as.character(v)
            p$labs$sub <- NULL

            if (nrow == 1 && scales %in% c("fixed", "free_x")) {
                graphics::par(mar = c(4, if (i == 1) 4 else 1.5, 3, 1))
            }

            bb_call_canvas(p$canvas, xlim = lim$xlim, ylim = lim$ylim)
            if (!is.null(p$panel.first)) {
                eval(p$panel.first())
            }

            for (ly in p$layer) {
                eval(ly())
            }
            do.call(title, p$labs)
        }

        if (length(outside_legends)) {
            graphics::par(mar = c(0, 0, 0, 0), xpd = NA)
            graphics::plot.new()
            for (j in seq_along(outside_legends)) {
                legend_obj <- outside_legends[[j]]
                legend_obj$outside <- FALSE
                legend_obj$position <- 0.05
                legend_obj$params$y <- 0.85 - (j - 1) * 0.25
                bb_draw_legend(legend_obj, x)
            }
        }

        if (!is.null(global_main)) {
            graphics::mtext(global_main, side = 3, outer = TRUE, line = if (!is.null(global_sub)) 1.5 else 1, font = 2)
        }
        if (!is.null(global_sub)) {
            graphics::mtext(global_sub, side = 3, outer = TRUE, line = 0.2)
        }

        .bbplot <- get_plotbb_env()
        assign(".last_plot", x, envir = .bbplot)
        return(invisible(x))
    }

    if (length(x$theme)) {
        old.par <- par(no.readonly=TRUE) #, new = TRUE)
        on.exit(suppressWarnings(par(old.par, no.readonly = TRUE)))
        par(x$theme, no.readonly = TRUE)
    }
    .bbplot <- get_plotbb_env()
    last_plot <- .bbplot$.last_plot

    if (!is.null(last_plot) && length(last_plot$theme))
        suppressWarnings(par(new = TRUE))

    lim <- bb_plot_limits(x)

    bb_call_canvas(x$canvas, xlim = lim$xlim, ylim = lim$ylim)
    if (!is.null(x$panel.first)) {
        eval(x$panel.first())
    } 
    
    

    for(ly in x$layer) {
        eval(ly())
    }

    do.call(title, x$labs)

    assign(".last_plot", x, envir = .bbplot)
}


##' @method print bbplot_layer_list
##' @export
print.bbplot_layer_list <- function(x, ...) {
    cat(length(x), "layers added to the plot\n")
    cat(paste0("  ", seq_along(x), ". ", names(x), collapse="\n"), "\n")
}

##' Facet a bbplot object into multiple panels.
##'
##' @title bb_facet_wrap
##' @param facets a formula specifying faceting variable, e.g. \code{~cyl}.
##' @param ncol number of columns.
##' @param nrow number of rows.
##' @param scales scale behavior across facets, one of 'fixed', 'free', 'free_x', or 'free_y'.
##' @return A modified bbplot object
##' @export
##' @author Guangchuang Yu
bb_facet_wrap <- function(facets, ncol = NULL, nrow = NULL, scales = "fixed") {
    stopifnot(inherits(facets, "formula"))
    scales <- match.arg(scales, c("fixed", "free", "free_x", "free_y"))
    structure(
        list(
            facets = facets,
            ncol = ncol,
            nrow = nrow,
            scales = scales
        ),
        class = "bb_facet_wrap"
    )
}

##' @method bbplot_add bb_facet_wrap
##' @export
bbplot_add.bb_facet_wrap <- function(object, plot) {
    plot$facet <- object
    plot
}

##' @rdname layer
##' @export
bb_tile <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ...,  layer = ly_tile)
}


ly_tile <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    ly <- function() {
        data$col <- bb_col(mapping, data, plot = plot)
        for (i in 1:nrow(data)) {
            rectangle(data$x[i], data$y[i], col = data$col[i])
        }
    }
    layer_name <- "heatmap layer"
    plot <- add_layer(plot, ly, layer_name)
    return(plot)
}

##' @importFrom graphics polygon
rectangle <- function(x, y, col, ...) {
    r0 = 0.5
    xx <- c(x - r0, x-r0, x + r0, x+ r0)
    yy <- c(y - r0, y + r0, y+r0, y - r0)
    polygon(xx, yy, col = col, ...)
}

##' histogram layer
##'
##' @title bb_hist
##' @rdname layer
##' @param mapping aesthetic mapping
##' @param data layer data
##' @param ... addition parameter for the layer
##' @return A modified bbplot object
##' @importFrom graphics hist
##' @export
##' @author Guangchuang Yu
bb_hist <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ..., layer = ly_hist)
}

ly_hist <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)
    
    params <- list(...)

    ly <- function() {
        if (is.null(mapping$x)) return(invisible(NULL))
        
        x <- bb_eval_one(mapping$x, data)
        args <- params
        
        # Handle mapped color (fill) and border?
        if (!is.null(mapping$col)) {
            # In base hist, 'col' is fill color. 'border' is border.
            # In ggplot, 'col' is border, 'fill' is fill.
            # Let's align with base plotbb style. 
            # bb_point uses 'col'.
            # Let's assume 'col' -> fill color for histograms if mimicking base?
            # Or 'col' -> border?
            # hist(..., col = "gray") fills it.
            
            col_vec <- bb_col(mapping, data, plot = plot)
            # hist doesn't support vector of colors easily if it's just one distribution?
            # actually `col` in hist can be a vector, "recycled as needed".
            # But if it's one histogram, we just want one color usually.
            # Unless we are stacking? bb_hist probably just wraps hist(), which is 1 variable.
            
            args$col <- col_vec
        }
        
        # Note: hist helper `add=TRUE` is needed if we are adding to existing plot
        # But bbplot creates a customized canvas.
        # If we use `bbplot() + bb_hist()`, the canvas is initialized to range of x/y.
        # Histogram has frequency on Y axis.
        # Standard `bbplot` might set Y limits based on data$y.
        # If we use `bb_hist`, y is count. Mapping doesn't know about it.
        # This is tricky in this architecture.
        # Use `plot = TRUE` to draw it?
        # If we just call `hist(..., add=TRUE)`, it fits on existing plot.
        # Does `bbplot` setup the correct coordinates?
        # If user does `bbplot(mtcars, bb_aes(mpg)) + bb_hist()`,
        # `bbplot` sees x=mpg, y=NULL.
        # `bbplot` implementation of default canvas:
        # xy <- bb_eval_xy(mapping, data)
        # args <- list(xy$x, xy$y, type='n', ...)
        # If y is NULL, `plot(x)` might do something different or fail if type='n' with missing y.
        # `plot(x, type='n')` works (index plot).
        
        # We need to handle `add=TRUE` inside `ly()`.
        
        args$add <- TRUE
        # We might need to ensure 'freq' or 'density' matches usage.
        
        do.call(graphics::hist, c(list(x = x), args))
    }
    
    add_layer(plot, ly, "histogram layer")
}

##' significance layer
##'
##' @title bb_signif
##' @rdname layer
##' @param mapping aesthetic mapping
##' @param data layer data
##' @param ... addition parameter for the layer
##' @return A modified bbplot object
##' @importFrom graphics segments
##' @importFrom graphics text
##' @export
##' @author Guangchuang Yu
bb_signif <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ..., layer = ly_signif)
}

ly_signif <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)
    
    params <- list(...)
    
    tip_length <- params$tip_length %||% 0.03
    params$tip_length <- NULL
    text_gap <- params$text_gap %||% 0.01
    params$text_gap <- NULL

    comparisons <- params$comparisons %||% NULL
    params$comparisons <- NULL

    y_position_param <- params$y_position %||% NULL
    params$y_position <- NULL

    annotations_param <- params$annotations %||% NULL
    params$annotations <- NULL

    step_increase <- params$step_increase %||% 0.05
    params$step_increase <- NULL

    test <- params$test %||% stats::wilcox.test
    params$test <- NULL

    test_args <- params$test_args %||% list()
    params$test_args <- NULL

    map_signif_level <- params$map_signif_level %||% TRUE
    params$map_signif_level <- NULL
    
    ly <- function() {
        usr <- par("usr")
        y_range <- usr[4] - usr[3]
        tip_h <- y_range * tip_length

        text_gap_abs <- y_range * text_gap

        if (!is.null(comparisons)) {
            x_raw <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
            y_raw <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))
            if (is.null(x_raw) || is.null(y_raw)) return(invisible(NULL))

            if (!is.list(comparisons)) comparisons <- list(comparisons)
            ncomp <- length(comparisons)
            if (ncomp == 0) return(invisible(NULL))

            if (is.numeric(x_raw)) {
                x_chr <- as.character(x_raw)
                lev <- NULL
                get_pos <- function(x) as.numeric(x)
                get_idx <- function(x) x_raw == as.numeric(x)
            } else {
                x_chr <- as.character(x_raw)
                lev <- bb_discrete_levels(x_raw)
                get_pos <- function(x) match(as.character(x), lev)
                get_idx <- function(x) x_chr == as.character(x)
            }

            if (is.null(y_position_param)) {
                y_span <- diff(range(y_raw, na.rm = TRUE))
                if (!is.finite(y_span) || y_span == 0) y_span <- 1
                y_top <- max(y_raw, na.rm = TRUE)
                y_position <- y_top + y_span * step_increase * seq_len(ncomp)
            } else if (length(y_position_param) == 1) {
                y_position <- rep(y_position_param, ncomp)
            } else {
                y_position <- y_position_param
            }

            xmin <- numeric(0)
            xmax <- numeric(0)
            y <- numeric(0)
            label <- character(0)

            resolve_test <- function(x) {
                if (is.function(x)) return(x)
                if (is.character(x)) {
                    x <- match.arg(x, c("t.test", "wilcox.test"))
                    if (identical(x, "t.test")) return(stats::t.test)
                    return(stats::wilcox.test)
                }
                stats::wilcox.test
            }
            test_fun <- resolve_test(test)
    if (identical(test_fun, stats::wilcox.test) && is.null(test_args$exact)) {
        test_args$exact <- FALSE
    }

    segments_params <- params
    segments_params$cex <- NULL
    segments_params$font <- NULL
    segments_params$family <- NULL

    text_params <- list(col = params$col, cex = params$cex, font = params$font, family = params$family)
    text_params <- text_params[!vapply(text_params, is.null, logical(1))]

            signif_label <- function(p) {
                if (is.null(p) || !is.finite(p)) return(NA_character_)
                if (!isTRUE(map_signif_level)) return(sprintf("p=%.3g", p))
                if (p <= 0.001) return("***")
                if (p <= 0.01) return("**")
                if (p <= 0.05) return("*")
                "ns"
            }

            for (i in seq_len(ncomp)) {
                comp <- comparisons[[i]]
                if (length(comp) < 2) next
                x1 <- comp[[1]]
                x2 <- comp[[2]]

                p1 <- get_pos(x1)
                p2 <- get_pos(x2)
                if (is.na(p1) || is.na(p2)) next

                idx1 <- get_idx(x1)
                idx2 <- get_idx(x2)
                y1 <- y_raw[idx1]
                y2 <- y_raw[idx2]

                pval <- NA_real_
                if (length(y1) && length(y2)) {
                    pval <- tryCatch(
                        do.call(test_fun, c(list(x = y1, y = y2), test_args))$p.value,
                        error = function(e) NA_real_
                    )
                }

                ann <- if (!is.null(annotations_param)) {
                    if (length(annotations_param) >= i) annotations_param[[i]] else NA_character_
                } else {
                    signif_label(pval)
                }

                xmin <- c(xmin, min(p1, p2))
                xmax <- c(xmax, max(p1, p2))
                y <- c(y, y_position[[i]])
                label <- c(label, ann)
            }

            if (length(xmin) == 0) return(invisible(NULL))

            do.call(graphics::segments, c(list(x0 = xmin, y0 = y, x1 = xmax, y1 = y), segments_params))
            do.call(graphics::segments, c(list(x0 = xmin, y0 = y, x1 = xmin, y1 = y - tip_h), segments_params))
            do.call(graphics::segments, c(list(x0 = xmax, y0 = y, x1 = xmax, y1 = y - tip_h), segments_params))

            if (length(label) && !all(is.na(label))) {
                x_mid <- (xmin + xmax) / 2
                do.call(graphics::text, c(list(x = x_mid, y = y + text_gap_abs, labels = label, pos = 3, offset = 0), text_params))
            }

            return(invisible(NULL))
        }

        xmin <- bb_eval_or_fallback(mapping, data, "xmin", NULL)
        xmax <- bb_eval_or_fallback(mapping, data, "xmax", NULL)
        y <- bb_eval_or_fallback(mapping, data, "y_position", NULL) %||% bb_eval_or_fallback(mapping, data, "y", NULL)
        label <- bb_eval_or_fallback(mapping, data, "label", NULL) %||% bb_eval_or_fallback(mapping, data, "annotations", NULL)
        if (is.null(xmin) || is.null(xmax) || is.null(y)) return(invisible(NULL))

        do.call(graphics::segments, c(list(x0 = xmin, y0 = y, x1 = xmax, y1 = y), segments_params))
        do.call(graphics::segments, c(list(x0 = xmin, y0 = y, x1 = xmin, y1 = y - tip_h), segments_params))
        do.call(graphics::segments, c(list(x0 = xmax, y0 = y, x1 = xmax, y1 = y - tip_h), segments_params))

        if (!is.null(label)) {
            x_mid <- (xmin + xmax) / 2
            do.call(graphics::text, c(list(x = x_mid, y = y + text_gap_abs, labels = label, pos = 3, offset = 0), text_params))
        }
    }
    
    add_layer(plot, ly, "signif layer")
}

##' layer
##'
##' bbplot layers
##' @title layer
##' @rdname layer
##' @param mapping aesthetic mapping
##' @param data layer data
##' @param position one of 'identity' or 'jitter'
##' @param ... addition parameter for the layer
##' @return A modified bbplot object
##' @importFrom graphics points
##' @export
##' @examples
##' library(plotbb)
##' p <- bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl))) +
##'   bb_point() + bb_lm(bb_aes(group=cyl), lwd=2)
##' @author Guangchuang Yu
bb_point <- function(mapping = NULL, data = NULL, position = "identity", ...) {
    build_layer(mapping, data, ...,  layer = ly_point)
}

ly_point <- function(plot, mapping = NULL, data = NULL, position = "identity", ...) {
    if (inherits(position, "bb_position")) {
        pos <- position$type %||% "identity"
    } else {
        pos <- match.arg(position, c("identity", "jitter"))
        position <- structure(list(type = pos), class = "bb_position")
    }

    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    xy <- bb_eval_xy(mapping, data)
    x <- xy$x
    y <- xy$y

    if (identical(position$type, "jitter")) {
        w <- position$width %||% NULL
        h <- position$height %||% NULL
        x <- jitter(x, amount = w)
        y <- jitter(y, amount = h)
    }

    params <- list(...)
    params <- modifyList(params, list(x = x, y = y))

    ly <- function() {
        if (!is.null(mapping$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
            params <- modifyList(params, list(col = col_vec))
        }

        if (!is.null(mapping$pch) && is.null(params$pch)) {
            pch_vec <- bb_pch(mapping, data, plot = plot)
            params <- modifyList(params, list(pch = pch_vec))
        }

        if (!is.null(mapping$cex) && is.null(params$cex)) {
            cex_vec <- bb_cex(mapping, data, plot = plot)
            params <- modifyList(params, list(cex = cex_vec))
        }

        do.call(points, params)
    }
    add_layer(plot, ly, "point layer")
}

##' @method bbplot_add bb_layer
##' @export
bbplot_add.bb_layer <- function(object, plot) {
    ly <- object$layer

    params_list <- object$params %||% list()
    object$params <- NULL
    object$layer <- NULL

    do.call(ly, c(list(plot = plot), object, params_list))
}

##' @rdname layer
##' @param se display confidence interval around smooth? (TRUE by default, see ggplot2)
##' @param level level of confidence interval to use (0.95 by default)
##' @export
bb_lm <- function(mapping = NULL, data = NULL, se = TRUE, level = 0.95, ...) {
    build_layer(mapping, data, se = se, level = level, ...,  layer = ly_lm)
}

##' @importFrom graphics segments polygon
##' @importFrom graphics par
ly_lm <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    layer_name <- "linear regression layer"
    formula <- stats::as.formula(paste(yvar(mapping), '~', xvar(mapping)))
    params <- list(...)
    se <- params$se %||% TRUE
    level <- params$level %||% 0.95

    if (is.null(mapping$group)) {
        d2 <- lm_data(formula, data, se = se, level = level)
        if (is.null(d2)) return(plot)
        ly <- function() {
            if (se && !is.null(d2$ci)) {
                ci <- d2$ci
                fill <- params$fill %||% "grey60"
                alpha <- params$alpha %||% 0.4
                graphics::polygon(c(ci$x, rev(ci$x)), c(ci$lwr, rev(ci$upr)), 
                                  col = scales::alpha(fill, alpha), border = NA)
            }
            args <- params
            args$se <- NULL
            args$level <- NULL
            args$fill <- NULL
            args$alpha <- NULL
            
            args$x0 <- d2$x0
            args$y0 <- d2$y0
            args$x1 <- d2$x1
            args$y1 <- d2$y1
            do.call(graphics::segments, args)
        }
        plot <- add_layer(plot, ly, layer_name)
        return(plot)
    }

    grp <- eval_mapping(mapping, 'group', data)
    ugrp <- unique(grp)

    ly <- function() {
        cols <- NULL
        if (is.null(params$col) && !is.null(mapping$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
            if (!is.null(col_vec)) {
                col_map <- tapply(col_vec, as.character(grp), function(z) z[[1]])
                cols <- unname(col_map[as.character(ugrp)])
            }
        }

        d2_list <- lapply(ugrp, function(g) {
            d <- data[grp == g, ]
            lm_data(formula, d, se = se, level = level)
        })
        keep <- !vapply(d2_list, is.null, logical(1))
        if (!any(keep)) return(invisible(NULL))
        
        d2_list <- d2_list[keep]
        if (!is.null(cols)) cols <- cols[keep]

        for (i in seq_along(d2_list)) {
            d2 <- d2_list[[i]]
            c_col <- if (!is.null(cols)) cols[i] else params$col %||% "black"
            
            if (se && !is.null(d2$ci)) {
                ci <- d2$ci
                fill <- params$fill %||% c_col
                alpha <- params$alpha %||% 0.4
                graphics::polygon(c(ci$x, rev(ci$x)), c(ci$lwr, rev(ci$upr)), 
                                  col = scales::alpha(fill, alpha), border = NA)
            }
            
            args <- params
            args$se <- NULL
            args$level <- NULL
            args$fill <- NULL
            args$alpha <- NULL
            
            args$x0 <- d2$x0
            args$y0 <- d2$y0
            args$x1 <- d2$x1
            args$y1 <- d2$y1
            args$col <- c_col
            
            do.call(graphics::segments, args)
        }
    }

    plot <- add_layer(plot, ly, layer_name)
    return(plot)
}

lm_data <- function(formula, data, se = FALSE, level = 0.95) {
    s <- tryCatch(stats::lm(formula, data = data), error = function(e) NULL)
    if (is.null(s)) return(NULL)
    
    term_labels <- attr(stats::terms(s), "term.labels")
    if (length(term_labels) != 1) return(NULL)

    mf <- stats::model.frame(s)
    x_name <- term_labels[[1]]
    if (!x_name %in% names(mf)) return(NULL)

    x <- mf[[x_name]]
    if (!is.numeric(x)) return(NULL)
    x <- x[is.finite(x)]
    if (length(x) == 0) return(NULL)

    co <- stats::coef(s)
    if (length(co) < 2 || any(!is.finite(co[1:2]))) return(NULL)

    x0 <- min(x)
    x1 <- max(x)
    y0 <- co[[1]] + x0 * co[[2]]
    y1 <- co[[1]] + x1 * co[[2]]
    
    res <- list(x0 = x0, x1 = x1, y0 = y0, y1 = y1)
    
    if (se && stats::df.residual(s) > 0) {
        x_seq <- seq(x0, x1, length.out = 80)
        nd <- data.frame(x_seq)
        names(nd) <- x_name
        pred <- tryCatch(stats::predict(s, newdata = nd, interval = "confidence", level = level), error = function(e) NULL)
        if (!is.null(pred)) {
            res$ci <- data.frame(x = x_seq, fit = pred[, "fit"], lwr = pred[, "lwr"], upr = pred[, "upr"])
        }
    }
    
    res
}

lm_env <- function(data) {
    lm_env <- new.env()
    lm_env$x0 <- data$x0
    lm_env$x1 <- data$x1
    lm_env$y0 <- data$y0
    lm_env$y1 <- data$y1
    if (!is.null(data$col))
        lm_env$col <- data$col
    return(lm_env)
}

## .ly_lm <- function(formula, data, col = par("fg"), lty = par("lty"), lwd = par("lwd")) {
##     s <- stats::lm(formula, data = data)
##     x <- as.character(formula)[3]

##     x0 <- min(data[[x]])
##     x1 <- max(data[[x]])
##     y0 <- s$coefficients[1] + x0 * s$coefficients[2]
##     y1 <- s$coefficients[1] + x1 * s$coefficients[2]
##     lm_env <- new.env()
##     lm_env$x0 <- x0
##     lm_env$x1 <- x1
##     lm_env$y0 <- y0
##     lm_env$y1 <- y1
##     lm_env$col <- col
##     lm_env$lty <- lty
##     lm_env$lwd <- lwd
##     ly <- function() segments(x0 = x0, y0 = y0,
##                               x1 = x1, y1 = y1,
##                               col = col, lty = lty,
##                               lwd = lwd)
##     with_env(ly, lm_env)
## }

##' @rdname layer
##' @export
bb_line <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ..., layer = ly_line)
}

ly_line <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    params <- list(...)

    xy <- bb_eval_xy(mapping, data)
    x <- xy$x
    y <- xy$y

    group <- eval_mapping(mapping, "group", data)

    ly <- function() {
        col_vec <- NULL
        if (is.null(params$col) && !is.null(mapping$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
        }

        if (!is.null(group)) {
            ug <- unique(group)
            cols <- NULL
            if (!is.null(col_vec)) {
                cols <- tapply(col_vec, group, function(z) z[[1]])
            }

            for (g in ug) {
                idx <- group == g
                args <- params
                args$x <- x[idx]
                args$y <- y[idx]
                if (!is.null(cols)) args$col <- unname(cols[[as.character(g)]])
                do.call(graphics::lines, args)
            }
            return(invisible(NULL))
        }

        args <- params
        if (!is.null(col_vec) && length(unique(col_vec)) == 1) {
            args$col <- col_vec[[1]]
        }
        args$x <- x
        args$y <- y
        do.call(graphics::lines, args)
        invisible(NULL)
    }
    add_layer(plot, ly, "line layer")
}

##' convert a base plot function to a bbplot object
##'
##' the base plot function will be plotted as a canvas and users can apply theme and add layers to it
##' @title as.bbplot
##' @rdname as-bbplot
##' @param fun a function that plot something in base graphics
##' @return A bbplot object
##' @export
##' @author Guangchuang Yu
as.bbplot <- function(fun) {
    stopifnot(is.function(fun))
    .bbplot_initial(fun)
}

.bbplot_initial <- function(canvas, data = NULL, mapping = bb_aes()) {
    p <- structure(list(
        canvas = canvas,
        data = data,
        mapping = mapping,
        scales = list(col = NULL),
        layer = structure(
            list(),
            class = "bbplot_layer_list"),
        theme = list(),
        env = new.env(),
        labs = list(main = NULL,
                    sub = NULL,
                    xlab = NULL,
                    ylab = NULL),
        panel.first = NULL,
        facet = NULL,
        adds = list()
    ), class = "bbplot")
    assign("palette", NULL, envir = p$env)
    assign("scales", list(col = NULL), envir = p$env)
    return(p)
}

##' bbplot 
##'
##' a proof of concept for grammar of graphics based on base plot.
##' The bbplot class contains data (input data), mapping (aesthetic mapping),
##' layer (a list of plot layers), theme (theme setting) and
##' labs (label setting, including title, subtitle, x and y labels).
##' @title bbplot
##' @param data data
##' @param mapping variable mapping
##' @return bbplot object
##' @importFrom graphics plot
##' @export
##' @examples
##' library(plotbb)
##' p <- bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl)))
##' p + bb_grid(col='grey50', lty='dashed') + bb_point(pch=19)
##' @author Guangchuang Yu
bbplot <- function(data, mapping = bb_aes()) {
    p <- function(xlim = NULL, ylim = NULL, ...) {
        xy <- bb_eval_xy(mapping, data)
        
        if (!is.null(xlim) && !is.null(ylim)) {
            args <- list(
                x = NA,
                y = NA,
                type = "n",
                xlab = "",
                ylab = "",
                xlim = xlim,
                ylim = ylim
            )
            do.call(graphics::plot, c(args, list(...)))
            return(invisible(NULL))
        }

        args <- list(
            xy$x %||% NA,
            xy$y %||% NA,
            type = "n",
            xlab = "",
            ylab = ""
        )
        if (!is.null(xlim)) args$xlim <- xlim
        if (!is.null(ylim)) args$ylim <- ylim
        do.call(graphics::plot, c(args, list(...)))
    }
    
    .bbplot_initial(p, data, mapping) +
        bb_labs(
            xlab = if (!is.null(mapping$x)) rlang::as_label(mapping$x) else NULL,
            ylab = if (!is.null(mapping$y)) rlang::as_label(mapping$y) else NULL
        )
}

get_plotbb_env <- function() {
    if(!exists(".bbplot")) .plotbb_env_initial()
    get(".bbplot")
}

with_env <- function(f, e=parent.frame()) {
    stopifnot(is.function(f))
    environment(f) <- e
    f
}


plot_fun <- function(x) {
    if (inherits(x, "formula")) {
        ## convert to expression
        x <- parse(text=as.character(x)[2])
    } 

    function() {
        if (inherits(x, "function"))
            return(x())
        eval(x)
    }
}


bb_data <- function(p, data) {
    data %||% p$data    
}

##' @importFrom utils modifyList
bb_mapping <- function(p, mapping) {
    if (!is.null(mapping)) {
        mapping <- modifyList(p$mapping, mapping)
    } else {
        mapping <- p$mapping
    }

    return(mapping)
}

xvar <- function(mapping) {
    get_mapping(mapping, 'x')
}

yvar <- function(mapping) {
    get_mapping(mapping, 'y')
}

##' @importFrom rlang quo_name
get_mapping <- function(mapping, name) {
    x <- mapping[[name]]
    if (is.null(x)) return(NULL)
    quo_name(x)
}

parse_mapping <- function(mapping, name, data) {
    v <- get_mapping(mapping, name)
    v2 <- gsub(".*\\((\\w+)\\).*", "\\1", v)
    eval(parse(text=sub(v2, "data[[v2]]", v)))
}

eval_mapping <- function(mapping, name, data) {
    x <- mapping[[name]]
    if (is.null(x)) return(NULL)
    rlang::eval_tidy(x, data = data)
}

bb_eval_or_fallback <- function(mapping, data, name, fallback_var = NULL) {
    v <- eval_mapping(mapping, name, data)
    if (!is.null(v)) return(v)
    if (!is.null(fallback_var)) return(data[[fallback_var]])
    NULL
}

bb_eval_xy <- function(mapping, data) {
    x <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
    y <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))
    list(x = x, y = y)
}

bb_resolve_palette <- function(plot, palette = NULL) {
    if (!is.null(palette)) return(palette)
    if (!is.null(plot) && !is.null(plot$env) && exists("palette", envir = plot$env, inherits = FALSE)) {
        return(get("palette", envir = plot$env, inherits = FALSE))
    }
    NULL
}

bb_call_canvas <- function(canvas, xlim = NULL, ylim = NULL) {
    fml <- formals(canvas)
    if (is.null(fml)) return(canvas())

    has_dots <- "..." %in% names(fml)
    args <- list()
    if (!is.null(xlim) && (has_dots || "xlim" %in% names(fml))) args$xlim <- xlim
    if (!is.null(ylim) && (has_dots || "ylim" %in% names(fml))) args$ylim <- ylim
    do.call(canvas, args)
}

bb_discrete_levels <- function(x) {
    if (is.null(x)) return(character())
    if (is.factor(x)) {
        lev <- levels(x)
        return(lev[!is.na(lev)])
    }
    chr <- as.character(x)
    chr <- chr[!is.na(chr)]
    sort(unique(chr))
}

bb_range_union <- function(rng, x) {
    if (is.null(x)) return(rng)
    if (length(x) == 0) return(rng)
    
    if (is.factor(x)) {
        x <- as.integer(x)
    }
    if (is.character(x)) {
        # Attempt to convert character to integer based on sorted unique levels (simple assumption)
        # This matches how we might treat discrete variables if no strict scale is enforced yet
        x <- as.integer(factor(x))
    }

    ok <- tryCatch(is.finite(x), error = function(e) NULL)
    if (is.null(ok)) return(rng)
    x <- x[ok]
    if (length(x) == 0) return(rng)
    xr <- range(x, na.rm = TRUE)
    if (is.null(rng)) return(xr)
    range(c(rng, xr), na.rm = TRUE)
}

bb_expand_range <- function(rng, mult = 0.04) {
    if (is.null(rng)) return(NULL)
    if (!all(is.finite(rng))) return(rng)
    span <- diff(rng)
    if (!is.finite(span) || span == 0) {
        pad <- ifelse(rng[[1]] == 0, 1, abs(rng[[1]]) * mult)
        return(rng + c(-pad, pad))
    }
    pad <- span * mult
    rng + c(-pad, pad)
}

bb_layer_limits <- function(layer_obj, plot) {
    if (!inherits(layer_obj, "bb_layer")) return(NULL)

    data <- bb_data(plot, layer_obj$data)
    mapping <- bb_mapping(plot, layer_obj$mapping)

    if (is.null(data) || is.null(mapping)) return(NULL)

    ly <- layer_obj$layer
    xlim <- NULL
    ylim <- NULL

    if (identical(ly, ly_segment)) {
        x <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
        y <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))
        xend <- bb_eval_or_fallback(mapping, data, "xend", NULL) %||% bb_eval_or_fallback(mapping, data, "x1", NULL)
        yend <- bb_eval_or_fallback(mapping, data, "yend", NULL) %||% bb_eval_or_fallback(mapping, data, "y1", NULL)
        xlim <- bb_range_union(xlim, x)
        xlim <- bb_range_union(xlim, xend)
        ylim <- bb_range_union(ylim, y)
        ylim <- bb_range_union(ylim, yend)
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_errorbar)) {
        orientation <- layer_obj$params$orientation %||% "y"
        width <- layer_obj$params$width %||% 0.5
        if (identical(orientation, "y")) {
            x <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
            ymin <- bb_eval_or_fallback(mapping, data, "ymin", NULL)
            ymax <- bb_eval_or_fallback(mapping, data, "ymax", NULL)
            xlim <- bb_range_union(xlim, x - width/2)
            xlim <- bb_range_union(xlim, x + width/2)
            ylim <- bb_range_union(ylim, ymin)
            ylim <- bb_range_union(ylim, ymax)
            return(list(xlim = xlim, ylim = ylim))
        }

        y <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))
        xmin <- bb_eval_or_fallback(mapping, data, "xmin", NULL)
        xmax <- bb_eval_or_fallback(mapping, data, "xmax", NULL)
        xlim <- bb_range_union(xlim, xmin)
        xlim <- bb_range_union(xlim, xmax)
        ylim <- bb_range_union(ylim, y - width/2)
        ylim <- bb_range_union(ylim, y + width/2)
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_lm)) {
        formula <- stats::as.formula(paste(yvar(mapping), "~", xvar(mapping)))
        grp <- eval_mapping(mapping, "group", data)
        se <- layer_obj$params$se %||% TRUE
        level <- layer_obj$params$level %||% 0.95
        if (is.null(grp)) {
            d2 <- lm_data(formula, data, se = se, level = level)
            if (is.null(d2)) return(NULL)
            xlim <- bb_range_union(xlim, c(d2$x0, d2$x1))
            ylim <- bb_range_union(ylim, c(d2$y0, d2$y1))
            if (se && !is.null(d2$ci)) {
                ylim <- bb_range_union(ylim, c(d2$ci$lwr, d2$ci$upr))
            }
            return(list(xlim = xlim, ylim = ylim))
        }

        grp_chr <- as.character(grp)
        keep <- !is.na(grp_chr)
        if (!any(keep)) return(NULL)
        for (g in unique(grp_chr[keep])) {
            d <- data[grp_chr == g, , drop = FALSE]
            d2 <- lm_data(formula, d, se = se, level = level)
            if (is.null(d2)) next
            xlim <- bb_range_union(xlim, c(d2$x0, d2$x1))
            ylim <- bb_range_union(ylim, c(d2$y0, d2$y1))
            if (se && !is.null(d2$ci)) {
                ylim <- bb_range_union(ylim, c(d2$ci$lwr, d2$ci$upr))
            }
        }
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_bar)) {
        stat <- layer_obj$params$stat %||% "identity"
        width <- layer_obj$params$width %||% 0.9

        x_raw <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
        if (identical(stat, "count")) {
            x_chr_all <- as.character(x_raw)
            x_chr <- x_chr_all[!is.na(x_chr_all)]
            if (length(x_chr) == 0) return(NULL)

            lev <- bb_discrete_levels(x_raw)
            tab <- table(factor(x_chr, levels = lev))
            xpos <- seq_along(tab)
            y <- as.numeric(tab)
            xlim <- bb_range_union(xlim, xpos - width/2)
            xlim <- bb_range_union(xlim, xpos + width/2)
            ylim <- bb_range_union(ylim, c(0, y))
            return(list(xlim = xlim, ylim = ylim))
        }

        y_raw <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))
        if (is.numeric(x_raw)) {
            xpos <- x_raw
        } else {
            labels <- unique(as.character(x_raw))
            xpos <- match(as.character(x_raw), labels)
        }
        xlim <- bb_range_union(xlim, xpos - width/2)
        xlim <- bb_range_union(xlim, xpos + width/2)
        ylim <- bb_range_union(ylim, c(0, y_raw))
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_boxplot)) {
        x <- bb_eval_or_fallback(mapping, data, "x", NULL)
        y <- bb_eval_or_fallback(mapping, data, "y", NULL)
        width <- layer_obj$params$boxwex %||% 0.8
        
        xlim <- NULL
        ylim <- NULL
        if (!is.null(x)) {
            if (is.factor(x) || is.character(x)) {
                xi <- as.integer(as.factor(x))
                xlim <- bb_range_union(xlim, xi - width/2)
                xlim <- bb_range_union(xlim, xi + width/2)
            } else {
                 xlim <- bb_range_union(xlim, x - width/2)
                 xlim <- bb_range_union(xlim, x + width/2)
            }
        } else {
            xlim <- c(1 - width/2, 1 + width/2)
        }
        ylim <- bb_range_union(ylim, y)
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_hist)) {
        x <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
        h <- graphics::hist(x, plot = FALSE)
        xlim <- bb_range_union(xlim, h$breaks)
        ylim <- bb_range_union(ylim, c(0, h$counts)) # Assuming counts for now
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_density)) {
        x <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
        if (!is.null(x) && is.numeric(x)) {
            grp <- eval_mapping(mapping, "col", data) %||% eval_mapping(mapping, "group", data) %||% eval_mapping(mapping, "fill", data)
            if (is.null(grp)) {
                d <- stats::density(x, na.rm = TRUE)
                xlim <- bb_range_union(xlim, d$x)
                ylim <- bb_range_union(ylim, c(0, d$y))
            } else {
                grp_chr <- as.character(grp)
                for (g in unique(grp_chr[!is.na(grp_chr)])) {
                    sub_x <- x[grp_chr == g]
                    if (length(sub_x) >= 2) {
                        d <- stats::density(sub_x, na.rm = TRUE)
                        xlim <- bb_range_union(xlim, d$x)
                        ylim <- bb_range_union(ylim, c(0, d$y))
                    }
                }
            }
        }
        return(list(xlim = xlim, ylim = ylim))
    }

    if (identical(ly, ly_signif)) {
        tip_length <- layer_obj$params$tip_length %||% 0.03
        text_gap <- layer_obj$params$text_gap %||% 0.01
        step_increase <- layer_obj$params$step_increase %||% 0.05

        comparisons <- layer_obj$params$comparisons %||% NULL
        y_position_param <- layer_obj$params$y_position %||% NULL

        if (!is.null(comparisons)) {
            x_raw <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
            y_raw <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))
            if (is.null(x_raw) || is.null(y_raw)) return(NULL)

            if (!is.list(comparisons)) comparisons <- list(comparisons)
            ncomp <- length(comparisons)
            if (ncomp == 0) return(NULL)

            y_span <- diff(range(y_raw, na.rm = TRUE))
            if (!is.finite(y_span) || y_span == 0) y_span <- 1

            if (is.null(y_position_param)) {
                y_top <- max(y_raw, na.rm = TRUE)
                y <- y_top + y_span * step_increase * seq_len(ncomp)
            } else if (length(y_position_param) == 1) {
                y <- rep(y_position_param, ncomp)
            } else {
                y <- y_position_param
            }

            if (is.numeric(x_raw)) {
                lev <- NULL
                get_pos <- function(x) as.numeric(x)
            } else {
                lev <- bb_discrete_levels(x_raw)
                get_pos <- function(x) match(as.character(x), lev)
            }

            xmin <- numeric(0)
            xmax <- numeric(0)
            for (i in seq_len(ncomp)) {
                comp <- comparisons[[i]]
                if (length(comp) < 2) next
                p1 <- get_pos(comp[[1]])
                p2 <- get_pos(comp[[2]])
                if (is.na(p1) || is.na(p2)) next
                xmin <- c(xmin, min(p1, p2))
                xmax <- c(xmax, max(p1, p2))
            }
            if (!length(xmin)) return(NULL)

            xlim <- bb_range_union(xlim, xmin)
            xlim <- bb_range_union(xlim, xmax)
            ylim <- bb_range_union(ylim, y - y_span * tip_length)
            ylim <- bb_range_union(ylim, y + y_span * text_gap)
            return(list(xlim = xlim, ylim = ylim))
        }

        xmin <- bb_eval_or_fallback(mapping, data, "xmin", NULL)
        xmax <- bb_eval_or_fallback(mapping, data, "xmax", NULL)
        y <- bb_eval_or_fallback(mapping, data, "y_position", NULL) %||% bb_eval_or_fallback(mapping, data, "y", NULL)
        if (is.null(xmin) || is.null(xmax) || is.null(y)) return(NULL)

        xlim <- bb_range_union(xlim, xmin)
        xlim <- bb_range_union(xlim, xmax)
        y_span <- diff(range(y, na.rm = TRUE))
        if (!is.finite(y_span) || y_span == 0) y_span <- 1
        ylim <- bb_range_union(ylim, y - y_span * tip_length)
        ylim <- bb_range_union(ylim, y + y_span * text_gap)
        return(list(xlim = xlim, ylim = ylim))
    }

    xy <- bb_eval_xy(mapping, data)
    xlim <- bb_range_union(xlim, xy$x)
    ylim <- bb_range_union(ylim, xy$y)
    list(xlim = xlim, ylim = ylim)
}

bb_eval_one <- function(quo, data) {
    if (is.null(quo)) return(NULL)
    rlang::eval_tidy(quo, data = data)
}


bb_plot_limits <- function(plot) {
    xlim <- NULL
    ylim <- NULL

    if (!is.null(plot$data) && !is.null(plot$mapping)) {
        xy <- bb_eval_xy(plot$mapping, plot$data)
        xlim <- bb_range_union(xlim, xy$x)
        ylim <- bb_range_union(ylim, xy$y)
    }

    if (length(plot$adds)) {
        for (obj in plot$adds) {
            lim <- bb_layer_limits(obj, plot)
            if (!is.null(lim$xlim)) xlim <- bb_range_union(xlim, lim$xlim)
            if (!is.null(lim$ylim)) ylim <- bb_range_union(ylim, lim$ylim)
        }
    }

    list(
        xlim = bb_expand_range(xlim),
        ylim = bb_expand_range(ylim)
    )
}

##' @importFrom scales col_numeric
##' @importFrom scales col_factor
bb_col <- function(mapping, data, plot = NULL, palette = NULL) {
    col_var <- eval_mapping(mapping, 'col', data)
    if (is.null(col_var)) return(NULL)

    if (!is.null(plot) && !is.null(plot$env) && exists("scales", envir = plot$env, inherits = FALSE)) {
        scales <- get("scales", envir = plot$env, inherits = FALSE)
        if (!is.null(scales$col)) {
            return(bb_scale_col_map(scales$col, col_var))
        }
    }

    if (!is.null(plot) && !is.null(plot$scales) && !is.null(plot$scales$col)) {
        return(bb_scale_col_map(plot$scales$col, col_var))
    }

    palette <- bb_resolve_palette(plot, palette)

    if (is.numeric(col_var)) {
        if (is.null(palette)) palette <- "viridis"
        f <- scales::col_numeric(palette, col_var)
        col_vec <- f(col_var)
    } else {
        ucol <- sort(unique(as.character(col_var)))
        if (is.null(palette)) palette <- "Set2"
        if (is.character(palette) && length(palette) > 1) {
            cols <- rep(palette, length.out = length(ucol))
            if (!is.null(names(palette))) {
                cols <- palette[ucol]
            }
            names(cols) <- ucol
            col_vec <- cols[as.character(col_var)]
        } else {
            f <- scales::col_factor(palette, ucol)
            cols <- f(ucol)
            names(cols) <- ucol
            col_vec <- cols[as.character(col_var)]
        }
    }
    return(col_vec)
}

bb_pch <- function(mapping, data, plot = NULL) {
    v <- eval_mapping(mapping, "pch", data)
    if (is.null(v)) return(NULL)

    if (!is.null(plot) && !is.null(plot$env) && exists("scales", envir = plot$env, inherits = FALSE)) {
        scales <- get("scales", envir = plot$env, inherits = FALSE)
        if (!is.null(scales$pch)) {
            out <- bb_scale_map(scales$pch, v)
            if (!is.null(out)) return(out)
        }
    }

    if (is.numeric(v)) return(as.integer(v))
    lev <- bb_discrete_levels(v)
    key <- as.character(v)
    pal <- rep(1:25, length.out = length(lev))
    names(pal) <- lev
    unname(pal[key])
}

bb_lty <- function(mapping, data, plot = NULL) {
    v <- eval_mapping(mapping, "lty", data)
    if (is.null(v)) return(NULL)

    if (!is.null(plot) && !is.null(plot$env) && exists("scales", envir = plot$env, inherits = FALSE)) {
        scales <- get("scales", envir = plot$env, inherits = FALSE)
        if (!is.null(scales$lty)) {
            out <- bb_scale_map(scales$lty, v)
            if (!is.null(out)) return(out)
        }
    }

    if (is.numeric(v)) return(v)
    lev <- bb_discrete_levels(v)
    key <- as.character(v)
    pal <- rep(1:6, length.out = length(lev))
    names(pal) <- lev
    unname(pal[key])
}

bb_cex <- function(mapping, data, plot = NULL) {
    v <- eval_mapping(mapping, "cex", data)
    if (is.null(v)) return(NULL)

    if (!is.null(plot) && !is.null(plot$env) && exists("scales", envir = plot$env, inherits = FALSE)) {
        scales <- get("scales", envir = plot$env, inherits = FALSE)
        if (!is.null(scales$cex)) {
            out <- bb_scale_map(scales$cex, v)
            if (!is.null(out)) return(out)
        }
    }

    if (is.numeric(v)) return(v)
    lev <- bb_discrete_levels(v)
    key <- as.character(v)
    pal <- rep(c(0.8, 1.1, 1.4, 1.7), length.out = length(lev))
    names(pal) <- lev
    unname(pal[key])
}


`%||%` <- function (a, b) {
    if (!is.null(a)) 
        a
    else b
}

##' @importFrom magrittr %>%
##' @export
magrittr::`%>%`


utils::globalVariables(".")

##' change col palette
##'
##' 
##' @title bb_scale_col_palette
##' @rdname bb-scale
##' @param palette color palette
##' @param values colors for discrete scale.
##' @param low low color for continuous scale.
##' @param high high color for continuous scale.
##' @return A modified bbplot object
##' @export
##' @author Guangchuang Yu
bb_scale_col_palette <- function(palette = NULL) {
    structure(list(palette = palette, type = "palette"), class = "bb_palette")
}

##' @rdname bb-scale
##' @export
bb_scale_col_manual <- function(values) {
    structure(list(palette = values, type = "manual"), class = "bb_palette")
}

##' @rdname bb-scale
##' @export
bb_scale_col_gradient <- function(low = "#132B43", high = "#56B1F7") {
    structure(list(palette = c(low, high), type = "gradient"), class = "bb_palette")
}

##' @title aesthetic scales
##' @rdname bb-scale-aes
##' @param values values used for mapping.
##' @return A modified bbplot object
##' @export
##' @author Guangchuang Yu
bb_scale_pch_manual <- function(values) {
    structure(list(aesthetic = "pch", type = "manual", values = values), class = "bb_scale")
}

##' @rdname bb-scale-aes
##' @export
bb_scale_lty_manual <- function(values) {
    structure(list(aesthetic = "lty", type = "manual", values = values), class = "bb_scale")
}

##' @rdname bb-scale-aes
##' @export
bb_scale_cex_manual <- function(values) {
    structure(list(aesthetic = "cex", type = "manual", values = values), class = "bb_scale")
}

##' @rdname bb-scale-aes
##' @param range numeric length-2 range for continuous mapping.
##' @export
bb_scale_cex_continuous <- function(range = c(0.8, 1.6)) {
    stopifnot(is.numeric(range), length(range) == 2)
    structure(list(aesthetic = "cex", type = "continuous", range = range), class = "bb_scale")
}

bb_scale_col_map <- function(scale, x) {
    if (is.null(scale)) return(NULL)
    type <- scale$type %||% "palette"
    palette <- scale$palette
    domain <- scale$domain

    if (type == "manual") {
        key <- as.character(x)
        ukey <- domain %||% bb_discrete_levels(key)
        cols <- rep(palette, length.out = length(ukey))
        if (!is.null(names(palette))) {
            cols <- palette[ukey]
        }
        names(cols) <- ukey
        return(cols[key])
    }

    if (type == "gradient") {
        f <- scales::col_numeric(palette, domain %||% x)
        return(f(x))
    }

    if (is.numeric(x)) {
        if (is.null(palette)) palette <- "viridis"
        f <- scales::col_numeric(palette, domain %||% x)
        return(f(x))
    }

    ucol <- domain %||% bb_discrete_levels(x)
    if (is.null(palette)) palette <- "Set2"
    if (is.character(palette) && length(palette) > 1) {
        cols <- rep(palette, length.out = length(ucol))
        if (!is.null(names(palette))) {
            cols <- palette[ucol]
        }
        names(cols) <- ucol
        return(cols[as.character(x)])
    }
    f <- scales::col_factor(palette, ucol)
    cols <- f(ucol)
    names(cols) <- ucol
    cols[as.character(x)]
}

bb_scale_map <- function(scale, x) {
    if (is.null(scale)) return(NULL)
    type <- scale$type %||% "manual"
    if (identical(type, "continuous")) {
        if (!is.numeric(x)) return(NULL)
        rng <- scale$range %||% c(0.8, 1.6)
        if (!is.numeric(rng) || length(rng) != 2) return(NULL)
        scales::rescale(x, to = rng, from = range(x, na.rm = TRUE))
    } else {
        key <- as.character(x)
        ukey <- scale$domain %||% bb_discrete_levels(key)
        vals <- scale$values
        if (is.null(vals)) return(NULL)
        mapped <- rep(vals, length.out = length(ukey))
        if (!is.null(names(vals))) {
            mapped <- vals[ukey]
        }
        names(mapped) <- ukey
        unname(mapped[key])
    }
}

##' @method bbplot_add bb_scale
##' @export
bbplot_add.bb_scale <- function(object, plot) {
    aesthetic <- object$aesthetic %||% NA_character_
    if (is.na(aesthetic) || !aesthetic %in% c("pch", "lty", "cex")) return(plot)

    if (is.null(plot$scales)) plot$scales <- list(col = NULL, pch = NULL, lty = NULL, cex = NULL)
    if (!exists("scales", envir = plot$env, inherits = FALSE)) {
        assign("scales", list(col = NULL, pch = NULL, lty = NULL, cex = NULL), envir = plot$env)
    }

    if (!is.null(plot$data) && !is.null(plot$mapping[[aesthetic]])) {
        v <- eval_mapping(plot$mapping, aesthetic, plot$data)
        if (!is.null(v)) {
            if (identical(object$type %||% "manual", "continuous")) {
                object$domain <- range(v, na.rm = TRUE)
            } else {
                object$domain <- bb_discrete_levels(v)
            }
        }
    }

    plot$scales[[aesthetic]] <- object
    scales <- get("scales", envir = plot$env, inherits = FALSE)
    scales[[aesthetic]] <- object
    assign("scales", scales, envir = plot$env)
    plot
}

##' Add legend for aesthetic mapping.
##'
##' @title bb_legend
##' @param position legend position passed to \code{graphics::legend}.
##' @param title legend title.
##' @param aesthetic currently supports \code{"col"}, \code{"fill"}, \code{"pch"}, \code{"lty"}, \code{"cex"}.
##' @param pch point shape used in legend.
##' @param bty box type passed to \code{graphics::legend}.
##' @param outside logical, whether to place the legend outside the plotting area.
##' @param ... additional parameters passed to \code{graphics::legend}.
##' @return A modified bbplot object
##' @export
##' @author Guangchuang Yu
bb_legend <- function(position = "topright", title = NULL, aesthetic = "col", pch = 19, bty = "n", outside = FALSE, ...) {
    structure(
        list(
            position = position,
            title = title,
            aesthetic = aesthetic,
            pch = pch,
            bty = bty,
            outside = outside,
            params = list(...)
        ),
        class = "bb_legend"
    )
}

bb_draw_legend <- function(object, plot) {
        mapping <- plot$mapping
        data <- plot$data
        if (is.null(data)) return(invisible(NULL))

        aesthetic <- object$aesthetic %||% "col"
        if (!aesthetic %in% c("col", "fill", "pch", "lty", "cex")) return(invisible(NULL))
        
        # fallback to col if fill is asked but mapping uses col
        map_quo <- mapping[[aesthetic]] %||% mapping[["col"]]
        if (is.null(map_quo)) return(invisible(NULL))

        aes_var <- eval_mapping(mapping, aesthetic, data) %||% eval_mapping(mapping, "col", data)
        if (is.null(aes_var)) return(invisible(NULL))

        legend_title <- object$title %||% rlang::as_label(map_quo)

        if (aesthetic %in% c("col", "fill") && is.numeric(aes_var)) {
            scale <- plot$scales[[aesthetic]] %||% plot$scales$col
            if (is.null(scale)) return(invisible(NULL))

            dom <- scale$domain %||% range(aes_var, na.rm = TRUE)
            if (length(dom) != 2 || any(!is.finite(dom))) return(invisible(NULL))

            pal <- scale$palette
            if (is.null(pal)) pal <- "viridis"
            f <- scales::col_numeric(pal, domain = dom)

            n <- 100
            z <- seq(dom[[1]], dom[[2]], length.out = n)
            cols <- f(z)

            usr <- par("usr")
            xspan <- usr[[2]] - usr[[1]]
            yspan <- usr[[4]] - usr[[3]]

            mar_x <- xspan * 0.02
            mar_y <- yspan * 0.02
            bar_w <- xspan * 0.05
            bar_h <- yspan * 0.45

            pos <- object$position %||% "topright"
            if (pos %in% c("topright", "right")) {
                x1 <- usr[[2]] - mar_x
                x0 <- x1 - bar_w
                y1 <- usr[[4]] - mar_y
                y0 <- y1 - bar_h
            } else if (pos %in% c("topleft", "left")) {
                x0 <- usr[[1]] + mar_x
                x1 <- x0 + bar_w
                y1 <- usr[[4]] - mar_y
                y0 <- y1 - bar_h
            } else if (pos %in% c("bottomright")) {
                x1 <- usr[[2]] - mar_x
                x0 <- x1 - bar_w
                y0 <- usr[[3]] + mar_y
                y1 <- y0 + bar_h
            } else if (pos %in% c("bottomleft")) {
                x0 <- usr[[1]] + mar_x
                x1 <- x0 + bar_w
                y0 <- usr[[3]] + mar_y
                y1 <- y0 + bar_h
            } else {
                x1 <- usr[[2]] - mar_x
                x0 <- x1 - bar_w
                y1 <- usr[[4]] - mar_y
                y0 <- y1 - bar_h
            }

            yy <- seq(y0, y1, length.out = n + 1)
            for (i in seq_len(n)) {
                graphics::rect(x0, yy[[i]], x1, yy[[i + 1]], col = cols[[i]], border = NA)
            }
            graphics::rect(x0, y0, x1, y1, border = "grey30")

            ticks <- pretty(dom, n = 5)
            ticks <- ticks[ticks >= dom[[1]] & ticks <= dom[[2]]]
            if (length(ticks)) {
                at <- y0 + (ticks - dom[[1]]) / (dom[[2]] - dom[[1]]) * (y1 - y0)
                graphics::axis(4, at = at, labels = ticks, las = 1, cex.axis = 0.8, tck = -0.02)
            }
            graphics::mtext(legend_title, side = 4, line = 2.5, cex = 0.8)
            return(invisible(NULL))
        }

        lvl <- as.character(aes_var)
        keep <- !is.na(lvl)
        if (!any(keep)) return(invisible(NULL))
        lvl <- lvl[keep]

        labels <- bb_discrete_levels(aes_var)
        labels <- labels[labels %in% lvl]
        if (length(labels) == 0) return(invisible(NULL))

        if (aesthetic %in% c("col", "fill")) {
            aes_vec <- bb_col(mapping, data, plot = plot)
            
            # Support mapping fill directly if no col map is defined or we explicitly requested fill
             if (!is.null(mapping$fill)) {
                  fill_var <- eval_mapping(mapping, "fill", data)
                  scale_fill <- plot$scales$fill %||% plot$scales$col
                  if (is.null(scale_fill)) {
                      scale_fill <- structure(list(palette = NULL, type = "palette"), class = "bb_palette")
                  }
                  mapped_cols <- bb_scale_col_map(scale_fill, fill_var)
                  if (!is.null(mapped_cols)) aes_vec <- mapped_cols
             }
            
            if (is.null(aes_vec)) return(invisible(NULL))
            aes_vec <- aes_vec[keep]
            
            if (length(aes_vec) != length(lvl)) return(invisible(NULL))
            
            aes_map <- tapply(aes_vec, lvl, function(z) z[[1]])
            aes_map <- unname(aes_map[labels])
            
            pch_val <- object$pch
            if (aesthetic == "fill" && pch_val == 19) pch_val <- 15
            
            args <- c(
                list(
                    x = object$position,
                    legend = labels,
                    col = aes_map,
                    pch = pch_val,
                    title = legend_title,
                    bty = object$bty
                ),
                object$params
            )
            
            if (object$outside) {
                op <- graphics::par(xpd = TRUE)
                on.exit(graphics::par(op), add = TRUE)
                if (is.character(args$x) && args$x %in% c("topright", "right", "bottomright")) {
                    args$inset <- c(-0.2, 0)
                }
            }
            
            do.call(graphics::legend, args)
            return(invisible(NULL))
        }

        if (identical(aesthetic, "pch")) {
            aes_vec <- bb_pch(mapping, data, plot = plot)
            aes_vec <- aes_vec[keep]
            aes_map <- tapply(aes_vec, lvl, function(z) z[[1]])
            aes_map <- unname(aes_map[labels])
            args <- c(
                list(
                    x = object$position,
                    legend = labels,
                    pch = aes_map,
                    title = legend_title,
                    bty = object$bty
                ),
                object$params
            )
            
            if (object$outside) {
                op <- graphics::par(xpd = TRUE)
                on.exit(graphics::par(op), add = TRUE)
                if (is.character(args$x) && args$x %in% c("topright", "right", "bottomright")) {
                    args$inset <- c(-0.2, 0)
                }
            }
            
            do.call(graphics::legend, args)
            return(invisible(NULL))
        }

        if (identical(aesthetic, "lty")) {
            aes_vec <- bb_lty(mapping, data, plot = plot)
            aes_vec <- aes_vec[keep]
            aes_map <- tapply(aes_vec, lvl, function(z) z[[1]])
            aes_map <- unname(aes_map[labels])
            args <- c(
                list(
                    x = object$position,
                    legend = labels,
                    lty = aes_map,
                    title = legend_title,
                    bty = object$bty
                ),
                object$params
            )
            
            if (object$outside) {
                op <- graphics::par(xpd = TRUE)
                on.exit(graphics::par(op), add = TRUE)
                if (is.character(args$x) && args$x %in% c("topright", "right", "bottomright")) {
                    args$inset <- c(-0.2, 0)
                }
            }
            
            do.call(graphics::legend, args)
            return(invisible(NULL))
        }

        aes_vec <- bb_cex(mapping, data, plot = plot)
        aes_vec <- aes_vec[keep]
        aes_map <- tapply(aes_vec, lvl, function(z) z[[1]])
        aes_map <- unname(aes_map[labels])
        args <- c(
            list(
                x = object$position,
                legend = labels,
                pch = object$pch,
                pt.cex = aes_map,
                title = legend_title,
                bty = object$bty
            ),
            object$params
        )
        
        if (object$outside) {
            op <- graphics::par(xpd = TRUE)
            on.exit(graphics::par(op), add = TRUE)
            if (is.character(args$x) && args$x %in% c("topright", "right", "bottomright")) {
                args$inset <- c(-0.2, 0)
            }
        }
        
        do.call(graphics::legend, args)
        invisible(NULL)
}

##' @method bbplot_add bb_legend
##' @export
bbplot_add.bb_legend <- function(object, plot) {
    ly <- function() {
        bb_draw_legend(object, plot)
    }

    add_layer(plot, ly, "legend")
}

##' @rdname layer
##' @param orientation one of 'y' or 'x'
##' @param width bar width or errorbar cap width
##' @export
bb_errorbar <- function(mapping = NULL, data = NULL, orientation = "y", width = 0.5, ...) {
    build_layer(mapping, data, ..., layer = ly_errorbar, orientation = orientation, width = width)
}

ly_errorbar <- function(plot, mapping = NULL, data = NULL, orientation = "y", width = 0.5, ...) {
    orientation <- match.arg(orientation, c("y", "x"))

    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    params <- list(...)

    if (orientation == "y") {
        x <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))

        ymin <- eval_mapping(mapping, "ymin", data)
        ymax <- eval_mapping(mapping, "ymax", data)
        if (is.null(ymin) || is.null(ymax)) stop("bb_errorbar requires ymin and ymax mapping")

        ly <- function() {
            col_vec <- NULL
            if (is.null(params$col) && !is.null(mapping$col)) {
                col_vec <- bb_col(mapping, data, plot = plot)
            }
            if (!is.null(col_vec)) {
                graphics::segments(x0 = x, y0 = ymin, x1 = x, y1 = ymax, col = col_vec, ...)
                graphics::segments(x0 = x - width/2, y0 = ymin, x1 = x + width/2, y1 = ymin, col = col_vec, ...)
                graphics::segments(x0 = x - width/2, y0 = ymax, x1 = x + width/2, y1 = ymax, col = col_vec, ...)
            } else {
                graphics::segments(x0 = x, y0 = ymin, x1 = x, y1 = ymax, ...)
                graphics::segments(x0 = x - width/2, y0 = ymin, x1 = x + width/2, y1 = ymin, ...)
                graphics::segments(x0 = x - width/2, y0 = ymax, x1 = x + width/2, y1 = ymax, ...)
            }
            invisible(NULL)
        }

        return(add_layer(plot, ly, "errorbar layer"))
    }

    y <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))

    xmin <- eval_mapping(mapping, "xmin", data)
    xmax <- eval_mapping(mapping, "xmax", data)
    if (is.null(xmin) || is.null(xmax)) stop("bb_errorbar requires xmin and xmax mapping")

    ly <- function() {
        col_vec <- NULL
        if (is.null(params$col) && !is.null(mapping$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
        }
        if (!is.null(col_vec)) {
            graphics::segments(x0 = xmin, y0 = y, x1 = xmax, y1 = y, col = col_vec, ...)
            graphics::segments(x0 = xmin, y0 = y - width/2, x1 = xmin, y1 = y + width/2, col = col_vec, ...)
            graphics::segments(x0 = xmax, y0 = y - width/2, x1 = xmax, y1 = y + width/2, col = col_vec, ...)
        } else {
            graphics::segments(x0 = xmin, y0 = y, x1 = xmax, y1 = y, ...)
            graphics::segments(x0 = xmin, y0 = y - width/2, x1 = xmin, y1 = y + width/2, ...)
            graphics::segments(x0 = xmax, y0 = y - width/2, x1 = xmax, y1 = y + width/2, ...)
        }
        invisible(NULL)
    }

    add_layer(plot, ly, "errorbar layer")
}

##' set bb_theme
##' 
##' setting bb_theme for ordinary base plot command. It internally use par to set global graphic parameters. 
##' Users need to explictely call unset_bb_theme() to restore original setting.
##' @title set_bb_theme
##' @param theme bb_theme
##' @return setting selected theme as default (has side effect and will affect other base plot)
##' @export
##' @examples
##' library(plotbb)
##' set_bb_theme(bb_theme_deepblue)
##' bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl))) + bb_point(pch=19)
##' @author Guangchuang Yu
set_bb_theme <- function(theme) {
    not_set <- is.null(getOption("bb_old_par"))
    if (not_set) {
        old.par <- par(no.readonly=TRUE) #, new = TRUE)
        options(bb_old_par = old.par)
    }
    if (is(theme, "function")) theme <- theme()
    par(theme, no.readonly = TRUE)
}

##' unset bb_theme
##' 
##' remove all the themes by set_bb_theme
##' @title unset_bb_theme
##' @return unset theme (i.e., restore par setting)
##' @export
##' @author Guangchuang Yu
unset_bb_theme <- function() {
    old.par <- getOption("bb_old_par")
    if (!is.null(old.par)) {
        suppressWarnings(par(old.par, no.readonly = TRUE))
    }
    invisible()
}


##' bbplot theme
##'
##' setting visual details of bbplot
##' @title bb_theme
##' @rdname bb-theme
##' @param ... parameters for graphics::par
##' @return A modified bbplot object
##' @export
##' @author Guangchuang Yu
bb_theme <- function(...) {
    default <- list(
        cex.axis = .9,
        las = 1
    )
    params <- modifyList(
        default,
        list(...)
    )
    structure(params, class = "bb_theme")
}



##' @method bbplot_add bb_theme
##' @export
bbplot_add.bb_theme <- function(object, plot) {
    plot$theme <- plot$theme <= object
    plot
}


##' @rdname bb-theme
##' @export
bb_theme_expand <- function(...) {
    bb_theme(
        mar=c(3,3,2,1),
        mgp=c(2,0.4,0),
        tck=-.01
    ) <= bb_theme(...)
}

##' @rdname bb-theme
##' @export
bb_theme_grey <- function(...) {
    bb_theme(
        bg = "grey85",
        fg = "grey20",
        col.axis = "grey20",
        col.lab = "grey20"
    ) <= bb_theme(...)
}

##' @rdname bb-theme
##' @export
bb_theme_deepblue <- function(...) {
    bb_theme(
        bg = "#002E49",
        fg = "#CCCCCC",
        col.axis = "#BEBEBE",
        col.lab = "#FFFFFF"
    ) <= bb_theme(...)
}

build_theme <- function(default, ...) {
    ## default is a list of default setting
    do.call(bb_theme, default) <= bb_theme(...)
}

##' @rdname bb-theme
##' @export
bb_theme_bw <- function(...) {
    bb_theme(
        bg = "white",
        fg = "black",
        col.axis = "black",
        col.lab = "black"
    ) <= bb_theme(...)
}

##' @rdname bb-theme
##' @export
bb_theme_minimal <- function(...) {
    bb_theme(
        bg = "white",
        fg = "black",
        col.axis = "black",
        col.lab = "black",
        bty = "n",
        tck = -0.01
    ) <= bb_theme(...)
}

##' density layer
##'
##' @title bb_density
##' @rdname layer
##' @param mapping aesthetic mapping
##' @param data layer data
##' @param ... addition parameter for the layer, such as \code{adjust}, \code{kernel}.
##' @return A modified bbplot object
##' @importFrom stats density
##' @export
##' @author Guangchuang Yu
bb_density <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ..., layer = ly_density)
}

ly_density <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)
    
    params <- list(...)

    ly <- function() {
        if (is.null(mapping$x)) return(invisible(NULL))
        
        x_raw <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))
        if (is.null(x_raw) || !is.numeric(x_raw)) return(invisible(NULL))
        
        grp <- eval_mapping(mapping, "col", data) %||% eval_mapping(mapping, "group", data) %||% eval_mapping(mapping, "fill", data)
        
        args <- params
        alpha <- args$alpha %||% 0.5
        
        if (is.null(grp)) {
            d <- stats::density(x_raw, na.rm = TRUE, 
                                adjust = args$adjust %||% 1, 
                                kernel = args$kernel %||% "gaussian")
            col <- bb_col(mapping, data, plot = plot) %||% "black"
            if (length(col) > 0) col <- col[1]
            
            graphics::polygon(c(min(d$x), d$x, max(d$x)), c(0, d$y, 0), col = scales::alpha(col, alpha), border = col)
        } else {
            grp_chr <- as.character(grp)
            keep <- !is.na(grp_chr) & !is.na(x_raw)
            if (!any(keep)) return(invisible(NULL))
            
            grp_chr <- grp_chr[keep]
            x_raw <- x_raw[keep]
            
            levs <- bb_discrete_levels(grp)
            levs <- levs[levs %in% unique(grp_chr)]
            
            cols <- bb_col(mapping, data, plot = plot)
            
            # Support mapping fill directly for polygon-based layers.
            if (!is.null(mapping$fill)) {
                 fill_var <- eval_mapping(mapping, "fill", data)
                 scale_fill <- plot$scales$fill %||% plot$scales$col
                 if (is.null(scale_fill)) {
                     scale_fill <- structure(list(palette = NULL, type = "palette"), class = "bb_palette")
                 }
                 mapped_cols <- bb_scale_col_map(scale_fill, fill_var)
                 if (!is.null(mapped_cols)) cols <- mapped_cols
            }
            
            if (!is.null(cols)) {
                # Fallback safely if dimensions don't match
                if (length(cols) != length(grp_chr)) {
                   cols <- rep(cols, length.out = length(grp_chr))
                }
                col_map <- unname(tapply(cols, grp_chr, function(z) z[[1]])[levs])
            } else {
                col_map <- rep("black", length(levs))
            }
            
            for (i in seq_along(levs)) {
                g <- levs[i]
                sub_x <- x_raw[grp_chr == g]
                if (length(sub_x) < 2) next
                
                d <- stats::density(sub_x, na.rm = TRUE, 
                                    adjust = args$adjust %||% 1, 
                                    kernel = args$kernel %||% "gaussian")
                c_col <- col_map[i]
                graphics::polygon(c(min(d$x), d$x, max(d$x)), c(0, d$y, 0), col = scales::alpha(c_col, alpha), border = c_col)
            }
        }
    }
    
    add_layer(plot, ly, "density layer")
}

##' @rdname layer
##' @export
bb_text <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ...,  layer = ly_text)
}

##' @importFrom graphics text
ly_text <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)
    xy <- bb_eval_xy(mapping, data)
    x <- xy$x
    y <- xy$y
    
    params <- list(...)
    params <- modifyList(params, list(x = x, y = y))

    label <- eval_mapping(mapping, 'label', data)
    if (!is.null(label) && is.null(params$label) && is.null(params$labels)) {
        params <- modifyList(params, list(label = label))
    }


    ly <- function() {
        if (!is.null(mapping$col) && is.null(params$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
            params <- modifyList(params, list(col = col_vec))
        }

        do.call(text, params)
    }

    add_layer(plot, ly, "text layer")
}

##' @rdname layer
##' @param stat one of 'identity' or 'count'
##' @param width bar width or errorbar cap width
##' @export
bb_bar <- function(mapping = NULL, data = NULL, stat = "identity", width = 0.9, ...) {
    build_layer(mapping, data, ..., layer = ly_bar, stat = stat, width = width)
}

ly_bar <- function(plot, mapping = NULL, data = NULL, stat = "identity", width = 0.9, ...) {
    stat <- match.arg(stat, c("identity", "count"))

    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)
    if (is.null(data)) stop("bb_bar requires data")

    x_raw <- bb_eval_or_fallback(mapping, data, "x", xvar(mapping))

    params <- list(...)

    if (stat == "count") {
        x_chr_all <- as.character(x_raw)
        x_chr <- x_chr_all[!is.na(x_chr_all)]
        if (length(x_chr) == 0) {
            plot$canvas <- function(xlim = NULL, ylim = NULL) {
                args <- list(NA, NA, type = "n", xlab = "", ylab = "", xaxt = "n")
                if (!is.null(xlim)) args$xlim <- xlim
                if (!is.null(ylim)) args$ylim <- ylim
                do.call(graphics::plot.default, args)
            }
            ly <- function() {
                invisible(NULL)
            }
            return(add_layer(plot, ly, "bar layer"))
        }

        lev <- bb_discrete_levels(x_raw)

        tab <- table(factor(x_chr, levels = lev))
        labels <- names(tab)
        xpos <- seq_along(tab)
        y <- as.numeric(tab)

        plot$canvas <- function(xlim = NULL, ylim = NULL) {
            xlim_default <- c(min(xpos - width/2), max(xpos + width/2))
            ylim_default <- c(0, max(y, na.rm = TRUE))
            xlim <- xlim %||% xlim_default
            ylim <- ylim %||% ylim_default
            graphics::plot.default(NA, NA, type = "n", xlim = xlim, ylim = ylim, xlab = "", ylab = "", xaxt = "n")
            graphics::axis(1, at = xpos, labels = labels)
        }

        ly <- function() {
            args <- c(
                list(
                    xleft = xpos - width/2,
                    ybottom = 0,
                    xright = xpos + width/2,
                    ytop = y
                ),
                params
            )

            if (is.null(args$col) && !is.null(mapping$col)) {
                col_var <- eval_mapping(mapping, "col", data)
                if (!is.null(col_var)) {
                    col_vec <- bb_col(mapping, data, plot = plot)
                    col_map <- tapply(col_vec, as.character(col_var), function(z) z[[1]])
                    col_fill <- unname(col_map[labels])
                    args$col <- col_fill
                }
            }

            do.call(graphics::rect, args)
            invisible(NULL)
        }

        return(add_layer(plot, ly, "bar layer"))
    }

    y_raw <- bb_eval_or_fallback(mapping, data, "y", yvar(mapping))

    if (!is.numeric(x_raw)) {
        x_chr <- as.character(x_raw)
        labels <- unique(x_chr)
        xpos <- match(x_chr, labels)
    } else {
        labels <- NULL
        xpos <- x_raw
    }
    y <- y_raw

    if (!is.null(labels)) {
        plot$canvas <- function(xlim = NULL, ylim = NULL) {
            xlim_default <- c(min(xpos - width/2), max(xpos + width/2))
            ylim_default <- range(c(0, y), na.rm = TRUE)
            xlim <- xlim %||% xlim_default
            ylim <- ylim %||% ylim_default
            graphics::plot.default(NA, NA, type = "n", xlim = xlim, ylim = ylim, xlab = "", ylab = "", xaxt = "n")
            graphics::axis(1, at = seq_along(labels), labels = labels)
        }
    }

    ly <- function() {
        args <- c(
            list(
                xleft = xpos - width/2,
                ybottom = 0,
                xright = xpos + width/2,
                ytop = y
            ),
            params
        )

        if (is.null(args$col) && !is.null(mapping$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
            if (!is.null(col_vec)) {
                args$col <- col_vec
            }
        }

        do.call(graphics::rect, args)
        invisible(NULL)
    }

    add_layer(plot, ly, "bar layer")
}

##' aesthetic mapping
##'
##'
##' aesthetic mapping for bbplot
##'
##' @title bb_aes
##' @param x x variable
##' @param y y variable
##' @param ... other mappings
##' @return enquo expression describing variables mapping to aesthetic attributes 
##' @importFrom rlang enquos
##' @export
##' @author Guangchuang Yu
bb_aes <- function(x, y, ...) {
    enquos(x = x, y = y, ..., .ignore_empty = "all")
}

##' @method + bbplot
##' @export
`+.bbplot` <- function(e1, e2) {
    if (is.null(e2)) return(e1)
    e1$adds[[length(e1$adds) + 1]] <- e2
    bbplot_add(e2, e1)
}

##' Add custom objects to bbplot
##'
##' This generic allows you to add your own methods for
##' adding custom objects to a bbplot object.
##'
##' @title bbplot_add
##' @param object An object to add to the plot
##' @param plot The bbplot object to add `object` to
##' @return A modified bbplot object
##' @keywords internal
##' @export
##' @author Guangchuang Yu
bbplot_add <- function(object, plot) {
    UseMethod("bbplot_add")
}



add_layer <- function(plot, layer, layer_name = "customized layer") {
    nlayer <- length(plot$layer) + 1
    plot$layer[[nlayer]] <- layer
    names(plot$layer)[nlayer] <- layer_name
    return(plot)
}

build_layer <- function(mapping, data, ..., layer) {
    structure(list(mapping = mapping,
                   data = data,
                   params = list(...),
                   layer = layer
                   ),
              class = "bb_layer")
}

##' @method <= bb_theme
##' @importFrom methods is
##' @export
`<=.bb_theme` <- function(e1, e2) {
    stopifnot(is(e2, "bb_theme"))
    modifyList(e1, e2)
}

##' @method bbplot_add bb_palette
##' @export
bbplot_add.bb_palette <- function(object, plot) {
    assign("palette", object$palette, envir = plot$env)
    if (is.null(plot$scales)) plot$scales <- list(col = NULL, pch = NULL, lty = NULL, cex = NULL)
    if (!exists("scales", envir = plot$env, inherits = FALSE)) {
        assign("scales", list(col = NULL, pch = NULL, lty = NULL, cex = NULL), envir = plot$env)
    }

    if (!is.null(plot$data) && !is.null(plot$mapping$col)) {
        col_var <- eval_mapping(plot$mapping, "col", plot$data)
        if (!is.null(col_var)) {
            if (is.numeric(col_var)) {
                object$domain <- range(col_var, na.rm = TRUE)
            } else {
                object$domain <- bb_discrete_levels(col_var)
            }
        }
    }

    plot$scales$col <- object
    scales <- get("scales", envir = plot$env, inherits = FALSE)
    scales$col <- object
    assign("scales", scales, envir = plot$env)
    return(plot)
}

##' @method bbplot_add formula
##' @export
bbplot_add.formula <- function(object, plot) {
    envir <- parent.frame()
    layer <- plot_fun(object)
    with_env(layer, envir)
    add_layer(plot, layer)
}

##' @method bbplot_add expression
##' @export
bbplot_add.expression <- bbplot_add.formula

##' @method bbplot_add function
##' @export
bbplot_add.function <- bbplot_add.formula

##' @rdname layer
##' @export
bb_segment <- function(mapping = NULL, data = NULL, ...) {
    build_layer(mapping, data, ..., layer = ly_segment)
}

ly_segment <- function(plot, mapping = NULL, data = NULL, ...) {
    data <- bb_data(plot, data)
    mapping <- bb_mapping(plot, mapping)

    xy <- bb_eval_xy(mapping, data)
    x <- xy$x
    y <- xy$y

    xend <- eval_mapping(mapping, "xend", data) %||% eval_mapping(mapping, "x1", data)
    yend <- eval_mapping(mapping, "yend", data) %||% eval_mapping(mapping, "y1", data)
    if (is.null(xend) || is.null(yend)) stop("bb_segment requires xend and yend mapping")

    params <- list(...)
    params <- modifyList(params, list(x0 = x, y0 = y, x1 = xend, y1 = yend))

    ly <- function() {
        args <- params
        if (is.null(args$col) && !is.null(mapping$col)) {
            col_vec <- bb_col(mapping, data, plot = plot)
            args$col <- col_vec
        }
        do.call(graphics::segments, args)
        invisible(NULL)
    }

    add_layer(plot, ly, "segment layer")
}

---
title: "Grammar of Graphics for base plot"
author:
  - name: "Guangchuang Yu"
    affiliation: "School of Basic Medical Sciences, Southern Medical University"
date: today
format:
  html:
    toc: true
execute:
  message: false
  warning: false
  fig-width: 12
  fig-height: 9
knitr:
  opts_chunk:
    out.width: "100%"
vignette: >
  %\VignetteIndexEntry{plotbb}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
r_libs <- Sys.getenv("R_LIBS")
if (nzchar(r_libs)) {
  libs <- strsplit(r_libs, .Platform$path.sep, fixed = TRUE)[[1]]
  libs <- libs[nzchar(libs)]
  .libPaths(c(libs, .libPaths()))
}
library(dplyr)
library(plotbb)
```

## Aesthetic mapping

`bb_aes()` for aesthetic mapping, that equivalents to `ggplot2::aes()`.

```{r plotbb-aes}
library(plotbb)

p <- bbplot(mtcars, bb_aes(mpg, disp, col = factor(cyl)))
p + bb_grid(col = "grey50", lty = "dashed") + bb_point(pch = 19)
```

## Geometric layer

```{r plotbb-layer}
#| fig-height: 4
p2 <- p + bb_point() + bb_lm(bb_aes(group = cyl), lwd = 2)
p3 <- p2 + bb_lm(col = "red", lwd = 3, lty = "dotted")
p4 <- p + bb_text(bb_aes(label = cyl), cex = 2)
aplot::plot_list(p2, p3, p4, ncol = 3, tag_levels = "A")
```

### Line, segment and errorbar

```{r plotbb-line-seg-errorbar}
d2 <- group_by(mtcars, cyl) %>%
  summarize(
    x = cyl[1],
    y = mean(disp),
    sd = sd(disp)
  )

g1 <- bbplot(d2, bb_aes(x, y)) +
  bb_line(col = "grey50") +
  bb_point(pch = 19, cex = 1.5) +
  bb_errorbar(bb_aes(ymin = y - sd, ymax = y + sd), width = 0.8)

seg <- data.frame(x = 15, y = 150, xend = 30, yend = 450)
g2 <- bbplot(seg, bb_aes(x, y)) +
  bb_point(pch = 19) +
  bb_segment(bb_aes(xend = xend, yend = yend), lwd = 3, col = "steelblue")

aplot::plot_list(g1, g2, ncol = 2, tag_levels = "A")
```

### Bar plot

```{r plotbb-bar}
mtcars$cyl <- factor(mtcars$cyl, levels = c(8, 4, 6))
bbplot(mtcars, bb_aes(cyl, col = cyl)) +
  bb_bar(stat = "count") +
  bb_legend(position = "topleft")
```

```{r plotbb-bar-identity}
dbar <- group_by(mtcars, cyl) %>%
  summarize(mpg = mean(mpg), .groups = "drop")
bbplot(dbar, bb_aes(cyl, mpg, col = cyl)) +
  bb_bar() +
  bb_legend(position = "topleft")
```

### Boxplot and histogram

```{r plotbb-boxplot}
bbplot(mtcars, bb_aes(factor(cyl), mpg)) +
  bb_boxplot(fill = "grey90")
```

```{r plotbb-hist}
bbplot(mtcars, bb_aes(mpg)) +
  bb_hist(col = "grey80", border = "white")
```

### Significance annotations

```{r plotbb-signif}
bbplot(mtcars, bb_aes(factor(cyl), mpg)) +
  bb_boxplot(fill = "grey90") +
  bb_signif(comparisons = list(c("4", "6"), c("4", "8")))
```

### Heatmap

```{r plotbb-heatmap}
#| fig-height: 12
df <- data.frame(
  x = rep(1:10, 12),
  y = rep(1:12, each = 10),
  values = rnorm(120, mean = 10, sd = 5),
  type = sample(LETTERS[1:5], 120, replace = TRUE),
  stringsAsFactors = FALSE
)

h1 <- bbplot(df, bb_aes(x, y, col = values)) + bb_tile() +
  bb_title("heatmap for continuous numerical values")
h2 <- bbplot(df, bb_aes(x, y, col = values)) + bb_tile() + bb_scale_col_palette("YlOrRd") +
  bb_title("applying a color palette")
h3 <- bbplot(df, bb_aes(x, y, col = type)) + bb_tile() +
  bb_title("heatmap for discrete categorical values")
h4 <- bbplot(df, bb_aes(x, y, col = values)) + bb_tile() + bb_text(col = "black") +
  bb_title("heatmap with text labels") + bb_theme_expand()
aplot::plot_list(h1, h2, h3, h4, ncol = 2, tag_levels = "A")
```

### Supported layers

+ [x] bb_grid
+ [x] bb_point
+ [x] bb_lm
+ [x] bb_text
+ [x] bb_tile
+ [x] bb_line
+ [x] bb_segment
+ [x] bb_errorbar
+ [x] bb_bar
+ [x] bb_boxplot
+ [x] bb_hist
+ [x] bb_facet_wrap
+ [x] bb_signif


## Setting labels

```{r plotbb-labs}
p2 + bb_labs(
  title = "hello",
  sub = "just for demo",
  xlab = "this is xlab",
  ylab = "this is ylab"
) +
  bb_title("hello world")
```

## Theme

```{r plotbb-theme}
#| fig-height: 6
g <- p2 +
  bb_theme(
    col.main = "red",
    cex.main = 2,
    mar = c(4, 4, 3, 1)
  ) +
  bb_title("applying graphics::par")
g2 <- p2 + bb_title("theme has no side effect")
aplot::plot_list(g, g2, ncol = 2, tag_levels = "A")
```

`bb_theme` has no side effect and will only apply to the `bbplot` object that it added to.
This is very important for developing pre-defined themes.

```{r plotbb-theme-expand}
#| fig-height: 6
p4 <- p3 + bb_theme_expand()
aplot::plot_list(p4, p3, ncol = 2, tag_levels = "A")
```

```{r plotbb-theme-grey}
p + bb_grid(col = "grey50", lty = "dashed") +
  bb_point(pch = 19) +
  bb_theme_expand() +
  bb_theme_grey()
```

```{r plotbb-theme-blue}
p + bb_point(pch = 19, cex = 2) +
  bb_theme_expand() +
  bb_theme_deepblue()
```

```{r plotbb-theme-bw-minimal}
#| fig-height: 6
g_bw <- p2 + bb_theme_expand() + bb_theme_bw() + bb_title("bb_theme_bw")
g_min <- p2 + bb_theme_expand() + bb_theme_minimal() + bb_title("bb_theme_minimal")
aplot::plot_list(g_bw, g_min, ncol = 2, tag_levels = "A")
```

### Supported themes

+ [x] `bb_theme_expand`
+ [x] `bb_theme_grey`
+ [x] `bb_theme_deepblue`
+ [x] `bb_theme_bw`
+ [x] `bb_theme_minimal`

## Scale


### Supported scales

+ [x] `bb_scale_col_palette`
+ [x] `bb_scale_col_manual`
+ [x] `bb_scale_col_gradient`
+ [x] `bb_scale_pch_manual`
+ [x] `bb_scale_lty_manual`
+ [x] `bb_scale_cex_manual`
+ [x] `bb_scale_cex_continuous`

Color scales can be added anywhere in the `+` chain and will affect both layers and legends.

```{r plotbb-scale-manual}
pal <- c(`4` = "#1b9e77", `6` = "#d95f02", `8` = "#7570b3")
bbplot(mtcars, bb_aes(mpg, disp, col = factor(cyl))) +
  bb_point(pch = 19) +
  bb_scale_col_manual(pal) +
  bb_legend()
```

```{r plotbb-scale-gradient}
bbplot(mtcars, bb_aes(mpg, disp, col = hp)) +
  bb_point(pch = 19) +
  bb_scale_col_gradient(low = "grey80", high = "firebrick") +
  bb_legend()
```

### Scales for pch, lty and cex

```{r plotbb-scale-aes}
#| fig-width: 16
p5 <- bbplot(mtcars, bb_aes(mpg, disp, col = factor(cyl), pch = factor(gear), cex = hp)) +
  bb_point() +
  bb_scale_cex_continuous(range = c(0.8, 2)) +
  bb_legend(aesthetic = "col", position = "topleft") +
  bb_legend(aesthetic = "pch", position = "bottomleft")

p6 <- bbplot(mtcars, bb_aes(mpg, disp, lty = factor(gear))) +
  bb_lm(lwd = 2) +
  bb_scale_lty_manual(values = c(`3` = "solid", `4` = "dashed", `5` = "dotted")) +
  bb_legend(aesthetic = "lty", position = "topleft")

aplot::plot_list(p5, p6, ncol = 2, tag_levels = "A")
```

## Legend

```{r plotbb-legend}
p + bb_point(pch = 19) + bb_legend()
```

## Facet

```{r plotbb-facet}
#| fig-height: 4
p + bb_point(pch = 19) + bb_facet_wrap(~cyl, ncol = 3)
```

```{r plotbb-facet-free}
#| fig-height: 4
p + bb_point(pch = 19) + bb_facet_wrap(~cyl, ncol = 3, scales = "free_y")
```

## Using existing code with plotbb

Suppose we have existing code to plot something:

```r
plot(mtcars$mpg, mtcars$disp)
abline(lm(disp ~ mpg, data = mtcars), col = "red")
```

We can wrap the codes into a function:

```{r}
f <- function() {
  plot(mtcars$mpg, mtcars$disp)
  abline(lm(disp ~ mpg, data = mtcars), col = "red")
}
```

Then we can convert it to a `bbplot` object. The plot produced by the function will be used as the canvas, and we can apply theme and add layers to it:

```{r}
library(dplyr)
d <- group_by(mtcars, cyl) %>%
  summarize(xm = mean(mpg), ym = mean(disp))

pp <- as.bbplot(f) +
  bb_theme_expand() +
  bb_theme_grey() +
  bb_lm(bb_aes(mpg, disp, group = cyl, col = factor(cyl)), data = mtcars, lwd = 2, lty = "dashed") +
  bb_point(bb_aes(xm, ym, col = factor(cyl)), data = d, pch = 19, cex = 2) +
  bb_title("hello plotbb") +
  bb_grid(col = "grey30", lty = "dashed")
```

As there are many features currently not available, `plotbb` supports adding layers using base graphics commands that you are already familiar with. Any base graphics commands in formula, expression or function can be served as layer to be added to a `bbplot` object.

```{r plotbb-base}
pp + (~points(30, 400, pch = 19, col = "red", cex = 3)) +
  ~text(30, 420, label = "have fun :)", col = "blue", cex = 1.2)
```

Here is another example to use `plotbb` with the `ape` package to visualize phylogenetic tree.

```{r plotbb-ape}
#| fig-height: 8.5
require(ape)
set.seed(2020 - 09 - 10)
x = rtree(10)

p = as.bbplot(
  function()
    plot(
      x,
      cex = 2,
      edge.width = 2,
      edge.color = "white",
      tip.color = "purple"
    )
)

p + bb_theme_expand() + bb_theme_deepblue() +
  (~nodelabels(node = 14, pie = matrix(rep(1, 4), 1), cex = 2)) +
  (~nodelabels(node = 15, pie = matrix(rep(1, 3), 1), cex = 2)) +
  (~nodelabels(node = 18, pie = matrix(rep(1, 5), 1), cex = 2)) +
  bb_labs(title = "phylogenetic tree plotted by base graphics") +
  bb_theme(col.main = "firebrick", cex.main = 2)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labs.R
\name{bb_labs}
\alias{bb_labs}
\alias{bb_title}
\alias{bb_sub}
\alias{bb_xlab}
\alias{bb_ylab}
\title{bb_labs}
\usage{
bb_labs(title = NULL, sub = NULL, xlab = NULL, ylab = NULL)

bb_title(title)

bb_sub(sub)

bb_xlab(xlab)

bb_ylab(ylab)
}
\arguments{
\item{title}{title}

\item{sub}{sub}

\item{xlab}{xlab}

\item{ylab}{ylab}
}
\value{
A modified bbplot object
}
\description{
change labels for bbplot
}
\details{
setting one or several of 'title', 'sub', 'xlab', and 'ylab'
}
\examples{
library(plotbb)
p <- bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl))) + bb_point(pch=19)
p + bb_labs(title = "hello", sub = "just for demo",
            xlab="this is xlab", ylab = "this is ylab") +
    bb_title("hello world") # last one rules
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bbplot.R
\name{bbplot}
\alias{bbplot}
\title{bbplot}
\usage{
bbplot(data, mapping = bb_aes())
}
\arguments{
\item{data}{data}

\item{mapping}{variable mapping}
}
\value{
bbplot object
}
\description{
bbplot
}
\details{
a proof of concept for grammar of graphics based on base plot.
The bbplot class contains data (input data), mapping (aesthetic mapping),
layer (a list of plot layers), theme (theme setting) and
labs (label setting, including title, subtitle, x and y labels).
}
\examples{
library(plotbb)
p <- bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl)))
p + bb_grid(col='grey50', lty='dashed') + bb_point(pch=19)
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\name{bb_scale_col_palette}
\alias{bb_scale_col_palette}
\alias{bb_scale_col_manual}
\alias{bb_scale_col_gradient}
\title{bb_scale_col_palette}
\usage{
bb_scale_col_palette(palette = NULL)

bb_scale_col_manual(values)

bb_scale_col_gradient(low = "#132B43", high = "#56B1F7")
}
\arguments{
\item{palette}{color palette}

\item{values}{colors for discrete scale.}

\item{low}{low color for continuous scale.}

\item{high}{high color for continuous scale.}
}
\value{
A modified bbplot object
}
\description{
change col palette
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{unset_bb_theme}
\alias{unset_bb_theme}
\title{unset_bb_theme}
\usage{
unset_bb_theme()
}
\value{
unset theme (i.e., restore par setting)
}
\description{
unset bb_theme
}
\details{
remove all the themes by set_bb_theme
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aes.R
\name{bb_aes}
\alias{bb_aes}
\title{bb_aes}
\usage{
bb_aes(x, y, ...)
}
\arguments{
\item{x}{x variable}

\item{y}{y variable}

\item{...}{other mappings}
}
\value{
enquo expression describing variables mapping to aesthetic attributes
}
\description{
aesthetic mapping
}
\details{
aesthetic mapping for bbplot
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid.R
\name{bb_grid}
\alias{bb_grid}
\title{bb_grid}
\usage{
bb_grid(col = "lightgray", lty = par("lty"), lwd = par("lwd"), bg = TRUE)
}
\arguments{
\item{col}{line color}

\item{lty}{line type}

\item{lwd}{line width}

\item{bg}{whether plot the grid lines as background}
}
\value{
A modified bbplot object
}
\description{
add grid lines
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{bb_facet_wrap}
\alias{bb_facet_wrap}
\title{bb_facet_wrap}
\usage{
bb_facet_wrap(facets, ncol = NULL, nrow = NULL, scales = "fixed")
}
\arguments{
\item{facets}{a formula specifying faceting variable, e.g. \code{~cyl}.}

\item{ncol}{number of columns.}

\item{nrow}{number of rows.}

\item{scales}{scale behavior across facets, one of 'fixed', 'free', 'free_x', or 'free_y'.}
}
\value{
A modified bbplot object
}
\description{
Facet a bbplot object into multiple panels.
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bb-bar.R, R/bb-boxplot.R, R/bb-density.R,
%   R/bb-errorbar.R, R/bb-hist.R, R/bb-line.R, R/bb-lm.R, R/bb-point.R,
%   R/bb-segment.R, R/bb-signif.R, R/bb-text.R, R/bb-tile.R
\name{bb_bar}
\alias{bb_bar}
\alias{bb_boxplot}
\alias{bb_density}
\alias{bb_errorbar}
\alias{bb_hist}
\alias{bb_line}
\alias{bb_lm}
\alias{bb_point}
\alias{bb_segment}
\alias{bb_signif}
\alias{bb_text}
\alias{bb_tile}
\title{bb_boxplot}
\usage{
bb_bar(mapping = NULL, data = NULL, stat = "identity", width = 0.9, ...)

bb_boxplot(mapping = NULL, data = NULL, ...)

bb_density(mapping = NULL, data = NULL, ...)

bb_errorbar(mapping = NULL, data = NULL, orientation = "y", width = 0.5, ...)

bb_hist(mapping = NULL, data = NULL, ...)

bb_line(mapping = NULL, data = NULL, ...)

bb_lm(mapping = NULL, data = NULL, se = TRUE, level = 0.95, ...)

bb_point(mapping = NULL, data = NULL, position = "identity", ...)

bb_segment(mapping = NULL, data = NULL, ...)

bb_signif(mapping = NULL, data = NULL, ...)

bb_text(mapping = NULL, data = NULL, ...)

bb_tile(mapping = NULL, data = NULL, ...)
}
\arguments{
\item{mapping}{aesthetic mapping}

\item{data}{layer data}

\item{stat}{one of 'identity' or 'count'}

\item{width}{bar width or errorbar cap width}

\item{...}{addition parameter for the layer}

\item{orientation}{one of 'y' or 'x'}

\item{se}{display confidence interval around smooth? (TRUE by default, see ggplot2)}

\item{level}{level of confidence interval to use (0.95 by default)}

\item{position}{one of 'identity' or 'jitter'}
}
\value{
A modified bbplot object

A modified bbplot object

A modified bbplot object

A modified bbplot object

A modified bbplot object
}
\description{
boxplot layer

density layer

histogram layer

layer

significance layer
}
\details{
bbplot layers
}
\examples{
library(plotbb)
p <- bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl))) +
  bb_point() + bb_lm(bb_aes(group=cyl), lwd=2)
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\name{bb_legend}
\alias{bb_legend}
\title{bb_legend}
\usage{
bb_legend(
  position = "topright",
  title = NULL,
  aesthetic = "col",
  pch = 19,
  bty = "n",
  outside = FALSE,
  ...
)
}
\arguments{
\item{position}{legend position passed to \code{graphics::legend}.}

\item{title}{legend title.}

\item{aesthetic}{currently supports \code{"col"}, \code{"fill"}, \code{"pch"}, \code{"lty"}, \code{"cex"}.}

\item{pch}{point shape used in legend.}

\item{bty}{box type passed to \code{graphics::legend}.}

\item{outside}{logical, whether to place the legend outside the plotting area.}

\item{...}{additional parameters passed to \code{graphics::legend}.}
}
\value{
A modified bbplot object
}
\description{
Add legend for aesthetic mapping.
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{bb_theme}
\alias{bb_theme}
\alias{bb_theme_expand}
\alias{bb_theme_grey}
\alias{bb_theme_deepblue}
\alias{bb_theme_bw}
\alias{bb_theme_minimal}
\title{bb_theme}
\usage{
bb_theme(...)

bb_theme_expand(...)

bb_theme_grey(...)

bb_theme_deepblue(...)

bb_theme_bw(...)

bb_theme_minimal(...)
}
\arguments{
\item{...}{parameters for graphics::par}
}
\value{
A modified bbplot object
}
\description{
bbplot theme
}
\details{
setting visual details of bbplot
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add-layer.R
\name{bbplot_add}
\alias{bbplot_add}
\title{bbplot_add}
\usage{
bbplot_add(object, plot)
}
\arguments{
\item{object}{An object to add to the plot}

\item{plot}{The bbplot object to add `object` to}
}
\value{
A modified bbplot object
}
\description{
Add custom objects to bbplot
}
\details{
This generic allows you to add your own methods for
adding custom objects to a bbplot object.
}
\author{
Guangchuang Yu
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as-bbplot.R
\name{as.bbplot}
\alias{as.bbplot}
\title{as.bbplot}
\usage{
as.bbplot(fun)
}
\arguments{
\item{fun}{a function that plot something in base graphics}
}
\value{
A bbplot object
}
\description{
convert a base plot function to a bbplot object
}
\details{
the base plot function will be plotted as a canvas and users can apply theme and add layers to it
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{set_bb_theme}
\alias{set_bb_theme}
\title{set_bb_theme}
\usage{
set_bb_theme(theme)
}
\arguments{
\item{theme}{bb_theme}
}
\value{
setting selected theme as default (has side effect and will affect other base plot)
}
\description{
set bb_theme
}
\details{
setting bb_theme for ordinary base plot command. It internally use par to set global graphic parameters. 
Users need to explictely call unset_bb_theme() to restore original setting.
}
\examples{
library(plotbb)
set_bb_theme(bb_theme_deepblue)
bbplot(mtcars, bb_aes(mpg, disp, col=factor(cyl))) + bb_point(pch=19)
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\name{bb_scale_pch_manual}
\alias{bb_scale_pch_manual}
\alias{bb_scale_lty_manual}
\alias{bb_scale_cex_manual}
\alias{bb_scale_cex_continuous}
\title{aesthetic scales}
\usage{
bb_scale_pch_manual(values)

bb_scale_lty_manual(values)

bb_scale_cex_manual(values)

bb_scale_cex_continuous(range = c(0.8, 1.6))
}
\arguments{
\item{values}{values used for mapping.}

\item{range}{numeric length-2 range for continuous mapping.}
}
\value{
A modified bbplot object
}
\description{
aesthetic scales
}
\author{
Guangchuang Yu
}
